from udsoncan import services, Request, Response
from udsoncan.connections import *
import struct
import time

conn = IsoTPConnection('vcan0', rxid=0x456, txid=0x123)
with conn.open():
    while True:
        payload = conn.wait_frame(timeout=None)
        if payload is not None:
            req = Request.from_payload(payload)
            response = Response(req.service, Response.Code.GeneralReject)

            # DiagnosticSessionControl
            if req.service == services.DiagnosticSessionControl:
                if req.subfunction == services.DiagnosticSessionControl.Session.extendedDiagnosticSession:
                    response = Response(
                        req.service, Response.Code.PositiveResponse, data=b'\x03')
                else:
                    response = Response(
                        req.service, Response.Code.SubFunctionNotSupported)

            # SecurityAccess
            elif req.service == services.SecurityAccess:
                if req.subfunction == 3:
                    response = Response(
                        req.service, Response.Code.PositiveResponse, data=b"\x03\x12\x34\x56\x78")
                elif req.subfunction == 4:
                    if req.data == b"\xed\xcb\xa9\x87":
                        response = Response(
                            req.service, Response.Code.PositiveResponse, data=b'\x04')
                    else:
                        response = Response(
                            req.service, Response.Code.InvalidKey)
                else:
                    response = Response(
                        req.service, Response.Code.SubFunctionNotSupported)
            # Tester present
            elif req.service == services.TesterPresent:
                response = Response(
                    req.service, Response.Code.PositiveResponse)

            # Read Data By identifier
            elif req.service == services.ReadDataByIdentifier:

                if req.data == b"\x00\x01":
                    response = Response(
                        req.service, Response.Code.PositiveResponse, data=b'\x00\x01\x12\x34')
                elif req.data == b"\x00\x02":
                    response = Response(
                        req.service, Response.Code.PositiveResponse, data=b'\x00\x02\x56\x78')
                elif req.data == b"\x00\x03":
                    response = Response(
                        req.service, Response.Code.PositiveResponse, data=b'\x00\x03\x9a\xbc')
                elif req.data == b"\x00\x01\x00\x02\x00\x03":
                    response = Response(req.service, Response.Code.PositiveResponse,
                                        data=b'\x00\x01\x12\x34\x00\x02\x56\x78\x00\x03\x9A\xBC\xDE\x0F')
                else:
                    response = Response(
                        req.service, Response.Code.RequestOutOfRange)

            # Write Data By identifier
            elif req.service == services.WriteDataByIdentifier:
                if req.data[0:2] in [b"\x00\x01", b"\x00\x02", b"\x00\x03", b'\xF1\x90']:
                    response = Response(
                        req.service, Response.Code.PositiveResponse, req.data[0:2])
                else:
                    response = Response(
                        req.service, Response.Code.RequestOutOfRange)

            elif req.service == services.ECUReset:
                if req.subfunction in [1, 2]:
                    response = Response(
                        req.service, Response.Code.PositiveResponse, data=struct.pack('B', req.subfunction))
                else:
                    response = Response(
                        req.service, Response.Code.SubFunctionNotSupported)

            else:
                response = Response(
                    req.service, Response.Code.ServiceNotSupported)

            if not response.positive or not req.suppress_positive_response:
                conn.send(response)
            else:
                print("Suppressing positive response.")



from udsoncan.connections import QueueConnection
from test.ThreadableTest import ThreadableTest
from udsoncan.client import Client

class ClientServerTest(ThreadableTest):
    def __init__(self, *args, **kwargs):
        ThreadableTest.__init__(self, *args, **kwargs)

    def setUp(self):
        self.conn = QueueConnection(name='unittest', mtu=4095)

    def clientSetUp(self):
        self.udsclient = Client(self.conn, request_timeout=0.2)
        self.udsclient.set_config('logger_name', 'unittest')
        self.udsclient.set_config('exception_on_invalid_response', True)
        self.udsclient.set_config('exception_on_unexpected_response', True)
        self.udsclient.set_config('exception_on_negative_response', True)

        self.udsclient.open()
        if hasattr(self, "postClientSetUp"):
            self.postClientSetUp()

    def clientTearDown(self):
        self.udsclient.close()

    def wait_request_and_respond(self, bytes):
        self.conn.touserqueue.get(timeout=0.2)
        self.conn.fromuserqueue.put(bytes) 



from udsoncan.exceptions import TimeoutException
from udsoncan import connections, Request, Response
import queue
import logging
import socket

class StubbedIsoTPSocket(object):
    conns = {}

    def __init__(self, name=None, timeout=1):
        self.bound = False
        self.interface=None
        self.rxid=None
        self.txid=None
        self.timeout = timeout

        self.queue_in = queue.Queue()	# Client reads from this queue. Other end is simulated

    def bind(self, interface, rxid, txid):
        self.interface = interface
        self.rxid = rxid
        self.txid = txid
        self.bound = True
        sockkey = (self.interface, self.rxid, self.txid)
        if sockkey not in StubbedIsoTPSocket.conns:
            StubbedIsoTPSocket.conns[sockkey] = dict()
        StubbedIsoTPSocket.conns[sockkey][id(self)] = self

    def close(self):
        self.bound=False
        sockkey = (self.interface, self.rxid, self.txid)
        if sockkey in StubbedIsoTPSocket.conns:
            if id(self) in StubbedIsoTPSocket.conns[sockkey]:
                del StubbedIsoTPSocket.conns[sockkey][id(self)]
        while not self.queue_in.empty():
            self.queue_in.get()

    def send(self, payload):
        target_sockkey = (self.interface, self.txid, self.rxid)
        if target_sockkey in StubbedIsoTPSocket.conns:
            for sockid in StubbedIsoTPSocket.conns[target_sockkey]:
                StubbedIsoTPSocket.conns[target_sockkey][sockid].queue_in.put(payload)

    def recv(self):
        try:
            payload = self.queue_in.get(block=True, timeout=self.timeout)
        except queue.Empty:
            raise socket.timeout 
        return payload





#!/usr/bin/env python
# -*- coding: utf-8 -*-

import inspect
import struct
import math

from udsoncan.exceptions import *
from udsoncan.Request import Request
from udsoncan.Response import Response

import logging, logging.config
from os import path
__default_log_config_file = path.join(path.dirname(path.abspath(__file__)), 'logging.conf')

def setup_logging(config_file = __default_log_config_file):
    """
    This function setup the logger accordingly to the module provided cfg file
    """
    try:
        logging.config.fileConfig(config_file)
    except Exception as e:
        logging.warning('Cannot load logging configuration from %s. %s:%s' % (config_file, e.__class__.__name__, str(e)))

#Define how to encode/decode a Data Identifier value to/from a binary payload
class DidCodec:
    """
    This class defines how to encode/decode a Data Identifier value to/from a binary payload.

    One should extend this class and override the ``encode``, ``decode``, ``__len__`` methods as they will be used
    to generate or parse binary payloads.

            - ``encode`` Must receive any Python object and must return a bytes payload
            - ``decode`` Must receive a bytes payload and may return any Python object
            - ``__len__`` Must return the length of the bytes payload or raise a ``DidCodec.ReadAllRemainingData`` to read the whole payload. Reading the whole payload is not a feature proposed by ISO-14229.

    If a data can be processed by a pack string, then this class may be used as is, without being extended.

    :param packstr: A pack string used with struct.pack / struct.unpack. 
    :type packstr: string
    """

    class ReadAllRemainingData(Exception):
        pass

    def __init__(self, packstr=None):
        self.packstr = packstr

    def encode(self, *did_value):
        if self.packstr is None:
            raise NotImplementedError('Cannot encode DID to binary payload. Codec has no "encode" implementation')

        return struct.pack(self.packstr, *did_value)

    def decode(self, did_payload):
        if self.packstr is None:
            raise NotImplementedError('Cannot decode DID from binary payload. Codec has no "decode" implementation')

        return struct.unpack(self.packstr, did_payload)

    #Must tell the size of the payload encoded or expected for decoding
    def __len__(self):
        if self.packstr is None:
            raise NotImplementedError('Cannot tell the payload size. Codec has no "__len__" implementation')
        return struct.calcsize(self.packstr)

    @classmethod
    def from_config(cls, didconfig):
        if isinstance(didconfig, cls):	#the given object already is a DidCodec instance
            return didconfig

        # The definition of the codec is a class. Returns an instance of this codec.
        if inspect.isclass(didconfig) and issubclass(didconfig, cls):	
            return didconfig()

        # It could be that the codec is in a dict. (for io_control)
        if isinstance(didconfig, dict) and 'codec' in didconfig:
            return cls.from_config(didconfig['codec'])

        # The codec can be defined by a struct pack/unpack string
        if isinstance(didconfig, str):
            if len(didconfig) == 0:
                raise ValueError("pack/unpack string given for Codec config should not be empty.")
            return cls(packstr = didconfig)

        raise ValueError('Given codec of type %s is not a valid DidCodec' % (type(didconfig)))

class AsciiCodec(DidCodec):
    def __init__(self, string_len=None):
        if string_len is None:
            raise ValueError("You must provide a string length to the AsciiCodec")
        self.string_len = string_len

    def encode(self, string_ascii):
        if len(string_ascii) != self.string_len:
            raise ValueError('String must be %d long' % self.string_len)
        return string_ascii.encode('ascii')

    def decode(self, string_bin):
        string_ascii = string_bin.decode('ascii')
        if len(string_ascii) != self.string_len:
            raise ValueError('Trying to decode a string of %d bytes but codec expects %d bytes' % (len(string_ascii), self.string_len))
        return string_ascii

    def __len__(self):
        return self.string_len


# Some standards, such as J1939, break down the 3-byte ID into 2-byte ID and 1-byte subtypes. 
class Dtc:
    """
    Defines a Diagnostic Trouble Code which consist of a 3-byte ID, a status, a severity and some diagnostic data.

    :param dtcid: The 3-byte ID of the DTC
    :type dtcid: int

    """
    class Format:
        """
        Provide a list of DTC formats and their indices. These values are used by the :ref:`The ReadDTCInformation<ReadDtcInformation>` when requesting a number of DTCs.		
        """
        ISO15031_6 = 0
        ISO14229_1 = 1
        SAE_J1939_73 = 2
        ISO11992_4 = 3

        @classmethod
        def get_name(cls, given_id):
            if given_id is None:
                return ""

            for member in inspect.getmembers(cls):
                if isinstance(member[1], int):
                    if member[1] == given_id:
                        return member[0]

            return None

    # DTC Status byte
    # This byte is an 8-bit flag indicating how much we are sure that a DTC is active.
    class Status:
        """
        Represents a DTC status which consists of 8 boolean flags (a byte). All flags can be set after instantiation without problems. 

        :param test_failed: DTC is no longer failed at the time of the request
        :type test_failed: bool

        :param test_failed_this_operation_cycle: DTC never failed on the current operation cycle.
        :type test_failed_this_operation_cycle: bool

        :param pending: DTC failed on the current or previous operation cycle.
        :type pending: bool

        :param confirmed: DTC is not confirmed at the time of the request.
        :type confirmed: bool

        :param test_not_completed_since_last_clear: DTC test has been completed since the last codeclear.
        :type test_not_completed_since_last_clear: bool

        :param test_failed_since_last_clear: DTC test failed at least once since last code clear.
        :type test_failed_since_last_clear: bool

        :param test_not_completed_this_operation_cycle: DTC test completed this operation cycle.
        :type test_not_completed_this_operation_cycle: bool

        :param warning_indicator_requested: Server is not requesting warningIndicator to be active.
        :type warning_indicator_requested: bool
        """

        def __init__(self, test_failed=False, test_failed_this_operation_cycle=False, pending=False, confirmed=False, test_not_completed_since_last_clear=False, test_failed_since_last_clear=False, test_not_completed_this_operation_cycle=False, warning_indicator_requested=False):
            self.test_failed 								= test_failed
            self.test_failed_this_operation_cycle 			= test_failed_this_operation_cycle
            self.pending 									= pending
            self.confirmed 									= confirmed
            self.test_not_completed_since_last_clear 		= test_not_completed_since_last_clear
            self.test_failed_since_last_clear 				= test_failed_since_last_clear
            self.test_not_completed_this_operation_cycle 	= test_not_completed_this_operation_cycle
            self.warning_indicator_requested 				= warning_indicator_requested

        def get_byte_as_int(self):	# Returns the status byte as an integer 
            byte = 0
            byte |= 0x1 	if self.test_failed else 0
            byte |= 0x2 	if self.test_failed_this_operation_cycle else 0
            byte |= 0x4 	if self.pending else 0
            byte |= 0x8 	if self.confirmed else 0
            byte |= 0x10 	if self.test_not_completed_since_last_clear else 0
            byte |= 0x20 	if self.test_failed_since_last_clear else 0
            byte |= 0x40 	if self.test_not_completed_this_operation_cycle else 0
            byte |= 0x80 	if self.warning_indicator_requested else 0

            return byte

        def get_byte(self):	# Returns the status byte in "bytes" format for payload creation
            return struct.pack('B', self.get_byte_as_int())

        def set_byte(self, byte):	# Set all the status flags from the status byte
            if not isinstance(byte, int) and not isinstance(byte, bytes):
                raise ValueError('Given byte must be an integer or bytes object.')

            if isinstance(byte, bytes):
                byte = struct.unpack('B', byte[0])

            self.test_failed 								= True if byte & 0x01 > 0 else False
            self.test_failed_this_operation_cycle 			= True if byte & 0x02 > 0 else False
            self.pending 									= True if byte & 0x04 > 0 else False
            self.confirmed 									= True if byte & 0x08 > 0 else False
            self.test_not_completed_since_last_clear 		= True if byte & 0x10 > 0 else False
            self.test_failed_since_last_clear 				= True if byte & 0x20 > 0 else False
            self.test_not_completed_this_operation_cycle	= True if byte & 0x40 > 0 else False
            self.warning_indicator_requested 				= True if byte & 0x80 > 0 else False

        @classmethod
        def from_byte(cls, byte):
            status = cls()
            status.set_byte(byte)
            return status

    # DTC Severity byte, it's a 3-bit indicator telling how serious a trouble code is.
    class Severity:
        """
        Represents a DTC severity which consists of 3 boolean flags. All flags can be set after instantiation without problems. 

        :param maintenance_only: This value indicates that the failure requests maintenance only
        :type maintenance_only: bool

        :param check_at_next_exit: This value indicates that the failure requires a check of the vehicle at the next halt.
        :type check_at_next_exit: bool

        :param check_immediately: This value indicates that the failure requires an immediate check of the vehicle.
        :type check_immediately: bool
        """
        def __init__(self, maintenance_only=False, check_at_next_exit=False, check_immediately=False):
            self.maintenance_only 		= maintenance_only
            self.check_at_next_exit 	= check_at_next_exit
            self.check_immediately 		= check_immediately

        def get_byte_as_int(self):	
            byte = 0
            byte |= 0x20 	if self.maintenance_only else 0
            byte |= 0x40 	if self.check_at_next_exit else 0
            byte |= 0x80 	if self.check_immediately else 0

            return byte

        def get_byte(self):
            return struct.pack('B', self.get_byte_as_int())

        def set_byte(self, byte):
            if not isinstance(byte, int) and not isinstance(byte, bytes):
                raise ValueError('Given byte must be an integer or bytes object.')

            if isinstance(byte, bytes):
                byte = struct.unpack('B', byte[0])

            self.maintenance_only 			= True if byte & 0x20 > 0 else False
            self.check_at_next_exit 		= True if byte & 0x40 > 0 else False
            self.check_immediately 			= True if byte & 0x80 > 0 else False

        @property
        def available(self):
            return True if self.get_byte_as_int() > 0 else False


    def __init__(self, dtcid):
        self.id = dtcid
        self.status = Dtc.Status()
        self.snapshots = []  		# . DID codec must be configured
        self.extended_data = [] 	
        self.severity = Dtc.Severity()
        self.functional_unit = None 	# Implementation specific (ISO 14229 D.4)
        self.fault_counter = None 		# Common practice is to detect a specific failure many times before setting the DTC active. This counter should tell the actual count.


    def __repr__(self):
        return '<DTC ID=0x%06x, Status=0x%02x, Severity=0x%02x at 0x%08x>' % (self.id, self.status.get_byte_as_int(), self.severity.get_byte_as_int(), id(self))

    # A snapshot data. Not defined by ISO14229 and implementation specific. 
    # To read this data, the client must have a DID codec set in its config.
    class Snapshot:
        record_number = None
        did = None
        data = None
        raw_data = b''

    # Extended data. Not defined by ISO14229 and implementation specific
    # Only raw data can be given to user.
    class ExtendedData:
        record_number = None
        raw_data = b''


class AddressAndLengthFormatIdentifier:
    """
    This class defines how many bytes of a memorylocation, composed of an address and a memorysize, should be encoded when sent over the underlying protocol.
    Mainly used by :ref:`ReadMemoryByAddress<ReadMemoryByAddress>`, :ref:`WriteMemoryByAddress<WriteMemoryByAddress>`, :ref:`RequestDownload<RequestDownload>` and :ref:`RequestUpload<RequestUpload>` services

    Defined by ISO-14229:2006, Annex G

    :param address_format: The number of bits on which an address should be encoded. Possible values are 8, 16, 24, 32, 40
    :type address_format: int

    :param memorysize_format: The number of bits on which a memory size should be encoded. Possible values are 8, 16, 24, 32
    :type memorysize_format: int

    """
    address_map = {
            8 	: 1,
            16 	: 2,
            24	: 3,
            32 	: 4,
            40	: 5
    }

    memsize_map = {
            8 : 1,
            16 : 2,
            24 : 3,
            32 : 4
    }

    def __init__(self, address_format, memorysize_format):
        if address_format not in self.address_map:
            raise ValueError('address_format must ba an integer selected from : %s ' % (self.address_map.keys()))

        if not isinstance(memorysize_format, int) or not isinstance(address_format, int):
            raise ValueError('memorysize_format and address_format must be integers')

        if memorysize_format not in self.memsize_map:
            raise ValueError('memorysize_format must be an integer selected from : %s' % (self.memsize_map.keys()))


        self.memorysize_format = memorysize_format
        self.address_format = address_format

    def get_byte_as_int(self):
        return  ((self.memsize_map[self.memorysize_format] << 4) | (self.address_map[self.address_format])) & 0xFF

    # Byte given alongside a memory address and a length so that they are decoded properly.
    def get_byte(self):
        return  struct.pack('B', self.get_byte_as_int())

class MemoryLocation:
    """
    This class defines a memory block location including : address, size, AddressAndLengthFormatIdentifier (address format and memory size format)

    :param address: A memory address pointing to the beginning of the memory block
    :type address: int

    :param memorysize: The size of the memory block
    :type memorysize: int

    :param address_format: The number of bits on which an address should be encoded. Possible values are 8, 16, 24, 32, 40.
            If ``None`` is specified, the smallest size required to store the given address will be used
    :type address_format: int or None

    :param memorysize_format: The number of bits on which a memory size should be encoded. Possible values are 8, 16, 24, 32
            If ``None`` is specified, the smallest size required to store the given memorysize will be used
    :type memorysize_format: int or None	

    """
    def __init__(self, address, memorysize, address_format=None, memorysize_format=None):
        self.address = address
        self.memorysize = memorysize
        self.address_format = address_format
        self.memorysize_format = memorysize_format

        if address_format is None:
            address_format = self.autosize_address(address)

        if memorysize_format is None:
            memorysize_format = self.autosize_memorysize(memorysize)

        self.alfid = AddressAndLengthFormatIdentifier(memorysize_format=memorysize_format, address_format=address_format)

    # This is used by the client/server to set a format from a config object while letting the user override it
    def set_format_if_none(self, address_format=None, memorysize_format=None):
        previous_address_format = self.address_format
        previous_memorysize_format = self.memorysize_format
        try:
            if address_format is not None:
                if self.address_format is None:
                    self.address_format = address_format

            if memorysize_format is not None:
                if self.memorysize_format is None:
                    self.memorysize_format=memorysize_format

            address_format = self.address_format if self.address_format is not None else self.autosize_address(self.address) 
            memorysize_format = self.memorysize_format if self.memorysize_format is not None else self.autosize_memorysize(self.memorysize) 

            self.alfid = AddressAndLengthFormatIdentifier(memorysize_format=memorysize_format, address_format=address_format)
        except:
            self.address_format = previous_address_format
            self.memorysize_format = previous_memorysize_format
            raise

    # Finds the smallest size that fits the address
    def autosize_address(self, val):
        fmt = math.ceil(val.bit_length()/8)*8
        if fmt > 40:
            raise ValueError("address size must be smaller or equal than 40 bits")
        return fmt

    # Finds the smallest size that fits the memory size
    def autosize_memorysize(self, val):
        fmt = math.ceil(val.bit_length()/8)*8
        if fmt > 32:
            raise ValueError("memory size must be smaller or equal than 32 bits")
        return fmt

    # Gets the address byte in the requested format
    def get_address_bytes(self):
        n = AddressAndLengthFormatIdentifier.address_map[self.alfid.address_format]

        data = struct.pack('>q', self.address)
        return data[-n:]


    # Gets the memory size byte in the requested format
    def get_memorysize_bytes(self):
        n = AddressAndLengthFormatIdentifier.memsize_map[self.alfid.memorysize_format]

        data = struct.pack('>q', self.memorysize)
        return data[-n:]

    # Generates an instance from the byte stream
    @classmethod
    def from_bytes(cls, address_bytes, memorysize_bytes):
        if not isinstance(address_bytes, bytes):
            raise ValueError('address_bytes must be a valid bytes object')

        if not isinstance(memorysize_bytes, bytes):
            raise ValueError('memorysize_bytes must be a valid bytes object')

        if len(address_bytes) > 5:
            raise ValueError('Address must be at most 40 bits long')

        if len(memorysize_bytes) > 4:
            raise ValueError('Memory size must be at most 32 bits long')

        address_bytes_padded = b'\x00' * (8-len(address_bytes)) + address_bytes
        memorysize_bytes_padded = b'\x00' * (8-len(memorysize_bytes)) + memorysize_bytes

        address = struct.unpack('>q', address_bytes_padded)[0]
        memorysize = struct.unpack('>q', memorysize_bytes_padded)[0]
        address_format = len(address_bytes) * 8
        memorysize_format = len(memorysize_bytes) * 8

        return cls(address=address, memorysize=memorysize, address_format=address_format, memorysize_format=memorysize_format)

    def __str__(self):
        return 'Address=0x%x (%d bits), Size=0x%x (%d bits)' % (self.address, self.alfid.address_format, self.memorysize, self.alfid.memorysize_format)

    def __repr__(self):
        return '<%s: %s at 0x%08x>' % (self.__class__.__name__, str(self), id(self))

class DataFormatIdentifier:
    """
    Defines the compression and encryption method of a specific chunk of data. 
    Mainly used by the :ref:`RequestUpload<RequestUpload>` and :ref:`RequestDownload<RequestDownload>` services

    :param compression: Value between 0 and 0xF specifying the compression method. Only the value 0 has a meaning defined by UDS standard and it is `No compression`. 
            All other values are ECU manufacturer specific.
    :type compression: int 

    :param encryption: Value between 0 and 0xF specifying the encryption method. Only the value 0 has a meaning defined by UDS standard and it is `No encryption`. 
            All other values are ECU manufacturer specific.
    :type encryption: int

    """
    def __init__(self, compression=0, encryption=0):
        both = (compression, encryption)
        for param in both:
            if not isinstance(param, int):
                raise ValueError('compression and encryption method must be an integer value')

            if param < 0 or param > 0xF:
                raise ValueError('compression and encryption method must each be an integer between 0 and 0xF')

        self.compression = compression
        self.encryption = encryption

    @classmethod
    def from_byte(cls, byte):
        byte = int(byte)
        return cls(compression = (byte >> 4)&0xF, encryption = (byte & 0xF) )

    def get_byte_as_int(self):
        return ((self.compression & 0xF) << 4) | (self.encryption & 0xF)

    def get_byte(self):
        return struct.pack('B', self.get_byte_as_int())

    def __str__(self):
        return 'Compression:0x%x, Encryption:0x%x' % (self.compression, self.encryption)

    def __repr__(self):
        return '<%s: %s at 0x%08x>' % (self.__class__.__name__, str(self), id(self))

# Units defined in standard. Nowhere does the ISO-14229 make use of them, but they are defined
class Units:
    #As defined in ISO-14229:2006 Annex C
    class Prefixs:
        class Prefix:
            def __init__(self, id, name, symbol, description=None):
                self.name = name
                self.id = id
                self.symbol = symbol
                self.description = description

            def __str__(self):
                return self.name

            def __repr__(self):
                desc = "(%s) " % self.description if self.description is not None else ""
                return "<UDS Unit prefix : %s[%s] %swith ID=%d at %08x>" % (self.name, self.symbol, desc, self.id, id(self))
        exa		= Prefix(id=0x40, name= 'exa', 	symbol='E', description='10e18')
        peta	= Prefix(id=0x41, name= 'peta', symbol='P', description='10e15')
        tera	= Prefix(id=0x42, name= 'tera', symbol='T', description='10e12')
        giga	= Prefix(id=0x43, name= 'giga', symbol='G', description='10e9')
        mega	= Prefix(id=0x44, name= 'mega', symbol='M', description='10e6')
        kilo	= Prefix(id=0x45, name= 'kilo', symbol='k', description='10e3')
        hecto	= Prefix(id=0x46, name= 'hecto', symbol='h', description='10e2')
        deca	= Prefix(id=0x47, name= 'deca', symbol='da', description='10e1')
        deci	= Prefix(id=0x48, name= 'deci', symbol='d', description='10e-1')
        centi	= Prefix(id=0x49, name= 'centi', symbol='c', description='10e-2')
        milli	= Prefix(id=0x4A, name= 'milli', symbol='m', description='10e-3')
        micro	= Prefix(id=0x4B, name= 'micro', symbol='m', description='10e-6')
        nano	= Prefix(id=0x4C, name= 'nano', symbol='n', description='10e-9')
        pico	= Prefix(id=0x4D, name= 'pico', symbol='p', description='10e-12')
        femto	= Prefix(id=0x4E, name= 'femto', symbol='f', description='10e-15')
        atto	= Prefix(id=0x4F, name= 'atto', symbol='a', description='10e-18')

    class Unit:
        def __init__(self, id, name, symbol, description=None):
            self.id =id
            self.name = name
            self.symbol = symbol
            self.description = description


        def __str__(self):
            return self.name

        def __repr__(self):
            desc = "(unit of %s) " % self.description if self.description is not None else ""
            return "<UDS Unit : %s[%s] %swith ID=%d at %08x>" % (self.name, self.symbol, desc, self.id, id(self))

    no_unit 			= Unit(id=0x00, name= 'no unit', 					symbol='-', 		description='-')
    meter 				= Unit(id=0x01, name= 'meter', 						symbol='m', 		description='length')
    foor 				= Unit(id=0x02, name= 'foot', 						symbol='ft', 		description='length')
    inch				= Unit(id=0x03, name= 'inch', 						symbol='in', 		description='length')
    yard				= Unit(id=0x04, name= 'yard', 						symbol='yd', 		description='length')
    english_mile		= Unit(id=0x05, name= 'mile (English)',				symbol='mi', 		description='length')
    gram				= Unit(id=0x06, name= 'gram', 						symbol='g', 		description='mass')
    metric_ton			= Unit(id=0x07, name= 'ton (metric)', 				symbol='t', 		description='mass')
    second				= Unit(id=0x08, name= 'second', 					symbol='s', 		description='time')
    minute				= Unit(id=0x09, name= 'minute', 					symbol='min', 		description='time')
    hour				= Unit(id=0x0A, name= 'hour', 						symbol='h', 		description='time')
    day					= Unit(id=0x0B, name= 'day', 						symbol='d', 		description='time')
    year				= Unit(id=0x0C, name= 'year', 						symbol='y', 		description='time')
    ampere				= Unit(id=0x0D, name= 'ampere', 					symbol='A', 		description='current')
    volt				= Unit(id=0x0E, name= 'volt', 						symbol='V', 		description='voltage')
    coulomb				= Unit(id=0x0F, name= 'coulomb', 					symbol='C', 		description='electric charge')
    ohm					= Unit(id=0x10, name= 'ohm', 						symbol='W', 		description='resistance')
    farad				= Unit(id=0x11, name= 'farad', 						symbol='F', 		description='capacitance')
    henry				= Unit(id=0x12, name= 'henry', 						symbol='H', 		description='inductance')
    siemens				= Unit(id=0x13, name= 'siemens', 					symbol='S', 		description='electric conductance')
    weber				= Unit(id=0x14, name= 'weber', 						symbol='Wb', 		description='magnetic flux')
    tesla				= Unit(id=0x15, name= 'tesla', 						symbol='T', 		description='magnetic flux density')
    kelvin				= Unit(id=0x16, name= 'kelvin', 					symbol='K', 		description='thermodynamic temperature')
    Celsius				= Unit(id=0x17, name= 'Celsius', 					symbol='°C', 		description='thermodynamic temperature')
    Fahrenheit			= Unit(id=0x18, name= 'Fahrenheit', 				symbol='°F', 		description='thermodynamic temperature')
    candela				= Unit(id=0x19, name= 'candela', 					symbol='cd', 		description='luminous intensity')
    radian				= Unit(id=0x1A, name= 'radian', 					symbol='rad', 		description='plane angle')
    degree				= Unit(id=0x1B, name= 'degree', 					symbol='°', 		description='plane angle')
    hertz				= Unit(id=0x1C, name= 'hertz', 						symbol='Hz', 		description='frequency')
    joule				= Unit(id=0x1D, name= 'joule', 						symbol='J', 		description='energy')
    Newton				= Unit(id=0x1E, name= 'Newton', 					symbol='N', 		description='force')
    kilopond			= Unit(id=0x1F, name= 'kilopond', 					symbol='kp', 		description='force')
    pound				= Unit(id=0x20, name= 'pound force', 				symbol='lbf', 		description='force')
    watt				= Unit(id=0x21, name= 'watt', 						symbol='W', 		description='power')
    horse				= Unit(id=0x22, name= 'horse power (metric)', 		symbol='hk', 		description='power')
    horse				= Unit(id=0x23, name= 'horse power(UK and US)', 	symbol='hp', 		description='power')
    Pascal				= Unit(id=0x24, name= 'Pascal', 					symbol='Pa', 		description='pressure')
    bar					= Unit(id=0x25, name= 'bar', 						symbol='bar', 		description='pressure')
    atmosphere			= Unit(id=0x26, name= 'atmosphere', 				symbol='atm', 		description='pressure')
    psi					= Unit(id=0x27, name= 'pound force per square inch',symbol='psi', 		description='pressure')
    becqerel			= Unit(id=0x28, name= 'becqerel', 					symbol='Bq', 		description='radioactivity')
    lumen				= Unit(id=0x29, name= 'lumen', 						symbol='lm', 		description='light flux')
    lux					= Unit(id=0x2A, name= 'lux', 						symbol='lx', 		description='illuminance')
    liter				= Unit(id=0x2B, name= 'liter', 						symbol='l', 		description='volume')
    gallon				= Unit(id=0x2C, name= 'gallon (British)', 			symbol='-', 		description='volume')
    gallon				= Unit(id=0x2D, name= 'gallon (US liq)', 			symbol='-', 		description='volume')
    cubic				= Unit(id=0x2E, name= 'cubic inch', 				symbol='cu in', 	description='volume')
    meter_per_sec		= Unit(id=0x2F, name= 'meter per seconds', 			symbol='m/s', 		description='speed')
    kmh					= Unit(id=0x30, name= 'kilometre per hour',			symbol='km/h', 		description='speed')
    mph					= Unit(id=0x31, name= 'mile per hour', 				symbol='mph', 		description='speed')
    rps					= Unit(id=0x32, name= 'revolutions per second', 	symbol='rps', 		description='angular velocity')
    rpm					= Unit(id=0x33, name= 'revolutions per minute', 	symbol='rpm', 		description='angular velocity')
    counts				= Unit(id=0x34, name= 'counts', 					symbol='-', 		description='-')
    percent				= Unit(id=0x35, name= 'percent', 					symbol='%', 		description='-')
    mg_per_stroke		= Unit(id=0x36, name= 'milligram per stroke', 		symbol='mg/stroke', description='mass per engine stroke')
    meter_per_sec2		= Unit(id=0x37, name= 'meter per square seconds', 	symbol='m/s2', 		description='acceleration')
    Nm					= Unit(id=0x38, name= 'Newton meter', 				symbol='Nm', 		description='moment')
    liter_per_min		= Unit(id=0x39, name= 'liter per minute', 			symbol='l/min', 	description='flow')
    watt_per_meter2		= Unit(id=0x3A, name= 'watt per square meter', 		symbol='W/m2', 		description='intensity')
    bar_per_sec			= Unit(id=0x3B, name= 'bar per second', 			symbol='bar/s', 	description='pressure change')
    radians_per_sec		= Unit(id=0x3C, name= 'radians per second', 		symbol='rad/s', 	description='angular velocity')
    radians				= Unit(id=0x3D, name= 'radians square second', 		symbol='rad/s2', 	description='angular acceleration')
    kilogram_per_meter2	= Unit(id=0x3E, name= 'kilogram per square meter', 	symbol='kg/m2', 	description='-')
    date1 				= Unit(id=0x50, name='Date1', 						symbol='-', 		description = 'Year-Month-Day')
    date2 				= Unit(id=0x51, name='Date2', 						symbol='-', 		description = 'Day/Month/Year')
    date3 				= Unit(id=0x52, name='Date3', 						symbol='-', 		description = 'Month/Day/Year')
    week 				= Unit(id=0x53, name='week', 						symbol='W', 		description = 'calendar week')
    time1 				= Unit(id=0x54, name='Time1', 						symbol='-', 		description = 'UTC Hour/Minute/Second')
    time2 				= Unit(id=0x55, name='Time2', 						symbol='-', 		description = 'Hour/Minute/Second')
    datetime1 			= Unit(id=0x56, name='DateAndTime1', 				symbol='-', 		description = 'Second/Minute/Hour/Day/Month/Year')
    datetime2 			= Unit(id=0x57, name='DateAndTime2', 				symbol='-', 		description = 'Second/Minute/Hour/Day/Month/Year/Local minute offset/Localhour offset')
    datetime3 			= Unit(id=0x58, name='DateAndTime3', 				symbol='-', 		description = 'Second/Minute/Hour/Month/Day/Year')
    datetime4 			= Unit(id=0x59, name='DateAndTime4', 				symbol='-', 		description = 'Second/Minute/Hour/Month/Day/Year/Local minute offset/Localhour offset')


# Routine class that containes few definitions for usage with nice syntax.
# myRoutine = Routine.EraseMemory    or    print(Routine.name_from_id(myRoutine))
class Routine:
    """
    Defines a list of constants that are routine identifiers defined by the UDS standard.
    This class provides no functionality apart from defining these constants
    """
    DeployLoopRoutineID	= 0xE200
    EraseMemory	= 0xFF00
    CheckProgrammingDependencies = 0xFF01
    EraseMirrorMemoryDTCs = 0xFF02


    @classmethod
    def name_from_id(cls, routine_id):
        # Helper to print the type of requests (logging purpose) as defined by ISO-14229:2006, Annex F
        if not isinstance(routine_id, int) or routine_id < 0 or routine_id > 0xFFFF:
            raise ValueError('Routine ID must be a valid integer between 0 and 0xFFFF')

        if routine_id >= 0x0000 and routine_id <= 0x00FF:
            return 'ISOSAEReserved'
        if routine_id >= 0x0100 and routine_id <= 0x01FF:
            return 'TachographTestIds'
        if routine_id >= 0x0200 and routine_id <= 0xDFFF:
            return 'VehicleManufacturerSpecific'
        if routine_id >= 0xE000 and routine_id <= 0xE1FF:
            return 'OBDTestIds'
        if routine_id == 0xE200:
            return 'DeployLoopRoutineID'
        if routine_id >= 0xE201 and routine_id <= 0xE2FF:
            return 'SafetySystemRoutineIDs'
        if routine_id >= 0xE300 and routine_id <= 0xEFFF:
            return 'ISOSAEReserved'
        if routine_id >= 0xF000 and routine_id <= 0xFEFF:
            return 'SystemSupplierSpecific'
        if routine_id == 0xFF00:
            return 'EraseMemory'
        if routine_id == 0xFF01:
            return 'CheckProgrammingDependencies'
        if routine_id == 0xFF02:
            return 'EraseMirrorMemoryDTCs'
        if routine_id >= 0xFF03 and routine_id <= 0xFFFF:
            return 'ISOSAEReserved'

class DataIdentifier:
    """
    Defines a list of constants that are data identifiers defined by the UDS standard.
    This class provides no functionality apart from defining these constants
    """
    BootSoftwareIdentification					= 0xF180
    ApplicationSoftwareIdentification			= 0xF181
    ApplicationDataIdentification				= 0xF182
    BootSoftwareFingerprint						= 0xF183
    ApplicationSoftwareFingerprint				= 0xF184
    ApplicationDataFingerprint					= 0xF185
    ActiveDiagnosticSession						= 0xF186
    VehicleManufacturerSparePartNumber			= 0xF187
    VehicleManufacturerECUSoftwareNumber		= 0xF188
    VehicleManufacturerECUSoftwareNumber		= 0xF188
    VehicleManufacturerECUSoftwareVersionNumber	= 0xF189
    SystemSupplierIdentifier					= 0xF18A
    ECUManufacturingDate						= 0xF18B
    ECUSerialNumber								= 0xF18C
    SupportedFunctionalUnits					= 0xF18D
    VehicleManufacturerKitAssemblyPartNumber	= 0xF18E
    ISOSAEReservedStandardized					= 0xF18F
    VIN											= 0xF190
    VehicleManufacturerECUHardwareNumber		= 0xF191
    SystemSupplierECUHardwareNumber				= 0xF192
    SystemSupplierECUHardwareVersionNumber		= 0xF193
    SystemSupplierECUSoftwareNumber				= 0xF194
    SystemSupplierECUSoftwareVersionNumber		= 0xF195
    ExhaustRegulationOrTypeApprovalNumber		= 0xF196
    SystemNameOrEngineType						= 0xF197
    RepairShopCodeOrTesterSerialNumber			= 0xF198
    ProgrammingDate								= 0xF199
    CalibrationRepairShopCodeOrCalibrationEquipmentSerialNumber	= 0xF19A
    CalibrationDate								= 0xF19B
    CalibrationEquipmentSoftwareNumber			= 0xF19C
    ECUInstallationDate							= 0xF19D
    ODXFile										= 0xF19E
    Entity										= 0xF19F

    @classmethod
    def name_from_id(cls, did):
        #As defined by ISO-14229:2006, Annex F
        if not isinstance(did, int) or did < 0 or did > 0xFFFF:
            raise ValueError('Data IDentifier must be a valid integer between 0 and 0xFFFF')

        if did >= 0x0000 and did <= 0x00FF:
            return 'ISOSAEReserved'
        if did >= 0x0100 and did <= 0xEFFF:
            return 'VehicleManufacturerSpecific'
        if did >= 0xF000 and did <= 0xF00F:
            return 'NetworkConfigurationDataForTractorTrailerApplicationDataIdentifier'
        if did >= 0xF010 and did <= 0xF0FF:
            return 'VehicleManufacturerSpecific'
        if did >= 0xF100 and did <= 0xF17F:
            return 'IdentificationOptionVehicleManufacturerSpecificDataIdentifier'

        if did == 0xF180:
            return 'BootSoftwareIdentificationDataIdentifier'
        if did == 0xF181:
            return 'ApplicationSoftwareIdentificationDataIdentifier'
        if did == 0xF182:
            return 'ApplicationDataIdentificationDataIdentifier'
        if did == 0xF183:
            return 'BootSoftwareFingerprintDataIdentifier'
        if did == 0xF184:
            return 'ApplicationSoftwareFingerprintDataIdentifier'
        if did == 0xF185:
            return 'ApplicationDataFingerprintDataIdentifier'
        if did == 0xF186:
            return 'ActiveDiagnosticSessionDataIdentifier'
        if did == 0xF187:
            return 'VehicleManufacturerSparePartNumberDataIdentifier'
        if did == 0xF188:
            return 'VehicleManufacturerECUSoftwareNumberDataIdentifier'
        if did == 0xF188:
            return 'VehicleManufacturerECUSoftwareNumberDataIdentifier'
        if did == 0xF189:
            return 'VehicleManufacturerECUSoftwareVersionNumberDataIdentifier'
        if did == 0xF18A:
            return 'SystemSupplierIdentifierDataIdentifier'
        if did == 0xF18B:
            return 'ECUManufacturingDateDataIdentifier'
        if did == 0xF18C:
            return 'ECUSerialNumberDataIdentifier'
        if did == 0xF18D:
            return 'SupportedFunctionalUnitsDataIdentifier'
        if did == 0xF18E:
            return 'VehicleManufacturerKitAssemblyPartNumberDataIdentifier'
        if did == 0xF18F:
            return 'ISOSAEReservedStandardized'
        if did == 0xF190:
            return 'VINDataIdentifier'
        if did == 0xF191:
            return 'VehicleManufacturerECUHardwareNumberDataIdentifier'
        if did == 0xF192:
            return 'SystemSupplierECUHardwareNumberDataIdentifier'
        if did == 0xF193:
            return 'SystemSupplierECUHardwareVersionNumberDataIdentifier'
        if did == 0xF194:
            return 'SystemSupplierECUSoftwareNumberDataIdentifier'
        if did == 0xF195:
            return 'SystemSupplierECUSoftwareVersionNumberDataIdentifier'
        if did == 0xF196:
            return 'ExhaustRegulationOrTypeApprovalNumberDataIdentifier'
        if did == 0xF197:
            return 'SystemNameOrEngineTypeDataIdentifier'
        if did == 0xF198:
            return 'RepairShopCodeOrTesterSerialNumberDataIdentifier'
        if did == 0xF199:
            return 'ProgrammingDateDataIdentifier'
        if did == 0xF19A:
            return 'CalibrationRepairShopCodeOrCalibrationEquipmentSerialNumberDataIdentifier'
        if did == 0xF19B:
            return 'CalibrationDateDataIdentifier'
        if did == 0xF19C:
            return 'CalibrationEquipmentSoftwareNumberDataIdentifier'
        if did == 0xF19D:
            return 'ECUInstallationDateDataIdentifier'
        if did == 0xF19E:
            return 'ODXFileDataIdentifier'
        if did == 0xF19F:
            return 'EntityDataIdentifier'

        if did >= 0xF1A0 and did <= 0xF1EF:
            return 'IdentificationOptionVehicleManufacturerSpecific'
        if did >= 0xF1F0 and did <= 0xF1FF:
            return 'IdentificationOptionSystemSupplierSpecific'
        if did >= 0xF200 and did <= 0xF2FF:
            return 'PeriodicDataIdentifier'	
        if did >= 0xF300 and did <= 0xF3FF:
            return 'DynamicallyDefinedDataIdentifier'
        if did >= 0xF400 and did <= 0xF4FF:
            return 'OBDDataIdentifier'
        if did >= 0xF500 and did <= 0xF5FF:
            return 'OBDDataIdentifier'
        if did >= 0xF600 and did <= 0xF6FF:
            return 'OBDMonitorDataIdentifier'
        if did >= 0xF700 and did <= 0xF7FF:
            return 'OBDMonitorDataIdentifier'
        if did >= 0xF800 and did <= 0xF8FF:
            return 'OBDInfoTypeDataIdentifier'
        if did >= 0xF900 and did <= 0xF9FF:
            return 'TachographDataIdentifier'
        if did >= 0xFA00 and did <= 0xFA0F:
            return 'AirbagDeploymentDataIdentifier'
        if did >= 0xFA10 and did <= 0xFAFF:
            return 'SafetySystemDataIdentifier'
        if did >= 0xFB00 and did <= 0xFCFF:
            return 'ReservedForLegislativeUse'
        if did >= 0xFD00 and did <= 0xFEFF:
            return 'SystemSupplierSpecific'
        if did >= 0xFF00 and did <= 0xFFFF:
            return 'ISOSAEReserved'

# Communication type is a single byte value including message type and subnet.
# Used by CommunicationControl service and defined by ISO-14229:2006 Annex B, table B.1
class CommunicationType:
    """
    This class represents a pair of subnet and message types. This value is mainly used by the :ref:`CommunicationControl<CommunicationControl>` service

    :param subnet: Represent the subnet number. Value ranges from 0 to 0xF 
    :type subnet: int

    :param normal_msg: Bit indicating that the `normal messages` are involved
    :type normal_msg: bool

    :param network_management_msg: Bit indicating that the `network management messages` are involved
    :type network_management_msg: bool

    """
    class Subnet:
        node = 0
        network = 0xF

        def __init__(self, subnet):
            if not isinstance(subnet, int):
                raise ValueError('subnet must be an integer value')

            if subnet < 0 or subnet > 0xF:
                raise ValueError('subnet must be an integer between 0 and 0xF')

            self.subnet=subnet

        def value(self):
            return self.subnet

    def __init__(self, subnet, normal_msg=False, network_management_msg=False):

        if not isinstance(subnet, self.Subnet):
            subnet = self.Subnet(subnet)

        if not isinstance(normal_msg, bool) or not isinstance(network_management_msg, bool):
            raise ValueError('message type (normal_msg, network_management_msg) must be valid boolean values')

        if normal_msg == False and network_management_msg == False:
            raise ValueError('At least one message type must be controlled')

        self.subnet = subnet
        self.normal_msg = normal_msg
        self.network_management_msg = network_management_msg

    def get_byte_as_int(self):
        message_type = 0
        if self.normal_msg:
            message_type |= 1
        if self.network_management_msg:
            message_type |= 2

        byte = (message_type & 0x3) | ((self.subnet.value() & 0xF) << 4)
        return byte

    def get_byte(self):
        return struct.pack('B', self.get_byte_as_int())

    @classmethod
    def from_byte(cls, val):
        if isinstance(val, bytes):
            val = struct.unpack('B', val)[0]
        val = int(val)
        subnet = (val & 0xF0) >> 4
        normal_msg = True if val & 1 > 0 else False
        network_management_msg = True if val & 2 > 0 else False
        return cls(subnet,normal_msg,network_management_msg)

    def __str__(self):
        flags = []
        if self.normal_msg:
            flags.append('NormalMsg')

        if self.network_management_msg:
            flags.append('NetworkManagementMsg')

        return 'subnet=0x%x. Flags : [%s]' % (self.subnet.value(), ','.join(flags))

    def __repr__(self):
        return '<%s: %s at 0x%08x>' % (self.__class__.__name__, str(self), id(self))

class Baudrate:
    """
    Represents a link speed in bit per seconds (or symbol per seconds to be more accurate).
    This class is used by the :ref:`LinkControl<LinkControl>` service that controls the underlying protocol speeds.

    The class can encode the baudrate in 2 different fashions : **Fixed** or **Specific**.

    Some standard baudrate values are defined within ISO-14229:2006 Annex B.3

    :param baudrate: The baudrate to be used. 
    :type baudrate: int

    :param baudtype: Tells how the baudrate shall be encoded. 4 values are possible:

            - ``Baudrate.Type.Fixed`` (0) : Will encode the baudrate in a single byte Fixed fashion. `baudrate` should be a supported value such as 9600, 19200, 125000, 250000, etc.
            - ``Baudrate.Type.Specific`` (1) : Will encode the baudrate in a three-byte Specific fashion. `baudrate` can be any value ranging from 0 to 0xFFFFFF
            - ``Baudrate.Type.Identifier`` (2) : Will encode the baudrate in a single byte Fixed fashion. `baudrate` should be the byte value to encode if the user wants to use a custom type.
            - ``Baudrate.Type.Auto`` (3) : Let the class guess the type. 

                    - If ``baudrate`` is a known standard value (19200, 38400, etc), then Fixed shall be used
                    - If ``baudrate`` is an integer that fits in a single byte, then Identifier shall be used
                    - If ``baudrate`` is none of the above, then Specific will be used.
    :type baudtype: int

    """
    baudrate_map = {
    9600 : 0x01,
    19200 : 0x02,
    38400 : 0x03,
    57600 : 0x04,
    115200 : 0x05,
    125000 : 0x10,
    250000 : 0x11,
    500000 : 0x12,
    1000000 : 0x13,
    }

    class Type:
        Fixed = 0		# When baudrate is a predefined value from standard
        Specific = 1	# When using custom baudrate
        Identifier = 2  # Baudrate implied by baudrate ID
        Auto = 3		# Let the class decide the type

    # User can specify the type of baudrate or let this class guess what he wants (this adds some simplicity for non-experts).
    def __init__(self, baudrate, baudtype=Type.Auto):
        if not isinstance(baudrate, int):
            raise ValueError('baudrate must be an integer')

        if baudrate < 0:
            raise ValueError('baudrate must be an integer greater than 0')

        if baudtype == self.Type.Auto:
            if baudrate in self.baudrate_map:
                self.baudtype = self.Type.Fixed
            else:
                if baudrate <= 0xFF:
                    self.baudtype = self.Type.Identifier
                else:
                    self.baudtype = self.Type.Specific
        else:
            self.baudtype = baudtype

        if self.baudtype == self.Type.Specific:
            if baudrate > 0xFFFFFF:
                raise ValueError('Baudrate value cannot be bigger than a 24 bits value.')

        elif self.baudtype == self.Type.Identifier:
            if baudrate > 0xFF:
                raise ValueError('Baudrate ID must be an integer between 0 and 0xFF')
        elif self.baudtype == self.Type.Fixed:
            if baudrate not in self.baudrate_map:
                raise ValueError('baudrate must be part of the supported baudrate list defined by UDS standard')
        else:
            raise ValueError('Unknown baudtype : %s' % self.baudtype)

        self.baudrate = baudrate

    # internal helper to change the type of this baudrate
    def make_new_type(self, baudtype):
        if baudtype not in [self.Type.Fixed, self.Type.Specific]:
            raise ValueError('Baudrate type can only be change to Fixed or Specific')

        return Baudrate(self.effective_baudrate(), baudtype=baudtype)

    # Returns the baudrate in Symbol Per Seconds if available, otherwise value given by the user.
    def effective_baudrate(self):
        if self.baudtype == self.Type.Identifier:
            for k in self.baudrate_map:
                if self.baudrate_map[k] == self.baudrate:
                    return k

            raise RuntimeError('Unknown effective baudrate, this could indicate a bug')
        else:
            return self.baudrate

    # Encodes the baudrate value the way they are exchanged.
    def get_bytes(self):
        if self.baudtype == self.Type.Fixed:
            return struct.pack('B', self.baudrate_map[self.baudrate])

        if self.baudtype == self.Type.Specific:
            b1 = (self.baudrate >> 16 ) & 0xFF
            b2 = (self.baudrate >> 8 ) & 0xFF
            b3 = (self.baudrate >> 0 ) & 0xFF
            return struct.pack('BBB', b1, b2, b3)

        if self.baudtype==self.Type.Identifier:
            return struct.pack('B', self.baudrate)

        raise RuntimeError('Unknown baudrate baudtype : %s' % self.baudtype)

    def __str__(self):
        baudtype_str = ''
        if self.baudtype == self.Type.Fixed:
            baudtype_str = 'Fixed'
        elif self.baudtype == self.Type.Specific:
            baudtype_str = 'Specific'
        elif self.baudtype == self.Type.Identifier:
            baudtype_str = 'Defined by identifier'

        return '%sBauds, %s format.' % (str(self.effective_baudrate()), baudtype_str)

    def __repr__(self):
        return '<%s: %s at 0x%08x>' % (self.__class__.__name__, str(self), id(self))

#Used for IO Control service. Allows comprehensive one-liner.
class IOMasks:
    """
    Allow to specify a list of masks for a :ref:`InputOutputControlByIdentifier<InputOutputControlByIdentifier>` composite codec.

    Example : IOMasks(mask1,mask2, mask3=True, mask4=False)

    :param args: Masks to set to True
    :param kwargs: Masks and their values
    """
    def __init__(self, *args, **kwargs):
        for k in kwargs:
            if not isinstance(kwargs[k], bool):
                raise ValueError('mask value must be a boolean value')

        for k in args:
            if not isinstance(k, str):
                raise ValueError('Mask name must be a valid string')

        self.maskdict = dict();
        for k in args:
            self.maskdict[k] = True

        for k in kwargs:
            if not isinstance(kwargs[k], bool):
                raise ValueError('Mask value must be True or False') 
            self.maskdict[k] = kwargs[k]

    def get_dict(self):
        return self.maskdict

#Used for IO Control service. Allows comprehensive one-liner.
class IOValues:
    """
    This class saves a function argument so they can be passed to a callback function.

    :param args: Arguments
    :param kwargs: Named arguments
    """
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs


class Filesize:
    """
    This class represent a file size used by the RequestFileTransfer service.

    :param uncompressed: Represent the uncompressed size in bytes
    :type uncompressed: int

    :param compressed: Represent the compressed size in bytes
    :type compressed: int

    :param width: The number of byte that should be used to encode the filesize in a payload
    :type width: int
    """
    def __init__(self, uncompressed=None, compressed = None, width=None):
        
        if uncompressed is None and compressed is None:
            raise ValueError('At least one size must be specified')

        if uncompressed is not None:
            if not isinstance(uncompressed, int):
                raise ValueError('Uncompressed size must be an integer')

            if uncompressed < 0:
                raise ValueError("Uncompressed size must be an integer greater than 0")

        if compressed is not None: 
            if not isinstance(compressed, int):
                raise ValueError('Compressed size must be an integer')
            
            if compressed < 0:
                raise ValueError("Comrpessed size must be an integer greater than 0")
        
        if width is not None:
            if not isinstance(width, int):
                raise ValueError('Width must be an integer')
    
            if width < 0:
                raise ValueError("Width size must be an integer greater than 0")
            maxsize = 2**(width*8)-1

            if compressed is not None:
                if compressed > maxsize:
                    raise ValueError("With width=%d, compressed size must be smaller than %d" % (width, maxsize))

            if uncompressed is not None:
                if uncompressed > maxsize:
                    raise ValueError("With width=%d, uncompressed size must be smaller than %d" % (width, maxsize))
        else:
            width_value = 0
            if uncompressed is not None:
                width_value = max(width_value, uncompressed)

            if compressed is not None:
                width_value = max(width_value, compressed)
            width = math.ceil(math.log2(width_value+1)/8)

        self.uncompressed = uncompressed
        self.compressed = compressed
        self.width = width

    def get_width(self):
        return self.width

    def get_uncompressed_bytes(self):
        if self.uncompressed is not None:
            return self.uncompressed.to_bytes(self.width, byteorder='big')
        else:
            return b''

    def get_compressed_bytes(self):
        if self.compressed is not None:
            return self.compressed.to_bytes(self.width, byteorder='big')
        else:
            return b''

    def __str__(self):
        uncompressed_str    = 'None' if self.uncompressed   is None else '0x%02x' % self.uncompressed
        compressed_str      = 'None' if self.compressed     is None else '0x%02x' % self.compressed
        width_str           = 'None' if self.width          is None else '0x%02x' % self.width

        return "Filesize<Uncompressed=%s, Compressed=%s. Width=%s>" % (uncompressed_str, compressed_str, width_str)

    def __repr__(self):
        uncompressed_str    = 'None' if self.uncompressed   is None else '0x%02x' % self.uncompressed
        compressed_str      = 'None' if self.compressed     is None else '0x%02x' % self.compressed
        width_str           = 'None' if self.width          is None else '0x%02x' % self.width

        return "<Filesize: Uncompressed=%s, Compressed=%s. Width=%s at 0x%08x>" % (uncompressed_str, compressed_str, width_str, id(self))


from test.UdsTest import UdsTest
import threading
import queue
import _thread as thread

#Class borrowed from Python Socket test suite.
class ThreadableTest(UdsTest):
    def __init__(self, *args, **kwargs):
        UdsTest.__init__(self, *args, **kwargs)
        # Swap the true setup function
        self.__setUp = self.setUp
        self.__tearDown = self.tearDown
        self.setUp = self._setUp
        self.tearDown = self._tearDown

    def serverExplicitReady(self):
        self.server_ready.set()

    def _setUp(self):
        self.server_ready = threading.Event()
        self.client_ready = threading.Event()
        self.done = threading.Event()
        self.queue = queue.Queue(1)
        self.server_crashed = False

        # Do some munging to start the client test.
        methodname = self.id()
        i = methodname.rfind('.')
        methodname = methodname[i+1:]
        test_method = getattr(self, '_' + methodname)
        self.client_thread = thread.start_new_thread(self.clientRun, (test_method,))

        try:
            self.__setUp()
        except:
            self.server_crashed = True
            raise
        finally:
            self.server_ready.set()
        self.client_ready.wait()

    def _tearDown(self):
        self.__tearDown()
        self.done.wait()

        if self.queue.qsize():
            exc = self.queue.get()
            raise exc

    def clientRun(self, test_func):
        self.server_ready.wait()
        try:
            self.clientSetUp()
        except BaseException as e:
            self.queue.put(e)
            self.client_ready.set()
            self._clientTearDown()
            return
        finally:
            self.client_ready.set()

        if self.server_crashed:
            self._clientTearDown()
            return
        if not hasattr(test_func, '__call__'):
            raise TypeError("test_func must be a callable function")
        try:
            test_func()
        except BaseException as e:
            self.queue.put(e)
        finally:
            self._clientTearDown()

    def clientSetUp(self):
        raise NotImplementedError("clientSetUp must be implemented.")

    def _clientTearDown(self):
        self.done.set()
        try:
            if hasattr(self, 'clientTearDown'):
                self.clientTearDown()
        except BaseException as e:
            self.queue.put(e)
        finally:			
            thread.exit()



from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *
import struct

class ClearDiagnosticInformation(BaseService):
    _sid = 0x14
    _use_subfunction = False
    _no_response_data = True

    supported_negative_response = [	 Response.Code.IncorrectMessageLegthOrInvalidFormat,
                                                    Response.Code.ConditionsNotCorrect,
                                                    Response.Code.RequestOutOfRange
                                                    ]

    @classmethod
    def make_request(cls, group=0xFFFFFF):
        """
        Generates a request for ClearDiagnosticInformation

        :param group: DTC mask ranging from 0 to 0xFFFFFF. 0xFFFFFF means all DTCs
        :type group: int

        :raises ValueError: If parameters are out of range, missing or wrong type
        """		
        from udsoncan import Request
        ServiceHelper.validate_int(group, min=0, max=0xFFFFFF, name='Group of DTC')
        request = Request(service=cls)
        hb = (group >> 16) & 0xFF
        mb = (group >> 8) & 0xFF
        lb = (group >> 0) & 0xFF 
        request.data = struct.pack("BBB", hb,mb,lb)
        return request

    @classmethod
    def interpret_response(cls, response):
        """
        Populates the response ``service_data`` property with an instance of :class:`ClearDiagnosticInformation.ResponseData<udsoncan.services.ClearDiagnosticInformation.ResponseData>`

        :param response: The received response to interpret
        :type response: :ref:`Response<Response>`
        """		
        response.service_data = cls.ResponseData()

    class ResponseData(BaseResponseData):
        """
        Empty object
        """		
        def __init__(self):
            super().__init__(ClearDiagnosticInformation)



from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *
import struct

class ReadDTCInformation(BaseService):
    _sid = 0x19

    supported_negative_response = [	 Response.Code.SubFunctionNotSupported,
                                                    Response.Code.IncorrectMessageLegthOrInvalidFormat,
                                                    Response.Code.RequestOutOfRange
                                                    ]

    class Subfunction(BaseSubfunction):
        __pretty_name__ = 'subfunction'

        reportNumberOfDTCByStatusMask = 1
        reportDTCByStatusMask = 2
        reportDTCSnapshotIdentification = 3
        reportDTCSnapshotRecordByDTCNumber = 4
        reportDTCSnapshotRecordByRecordNumber = 5
        reportDTCExtendedDataRecordByDTCNumber = 6
        reportNumberOfDTCBySeverityMaskRecord = 7
        reportDTCBySeverityMaskRecord = 8
        reportSeverityInformationOfDTC = 9
        reportSupportedDTCs = 0xA
        reportFirstTestFailedDTC = 0xB
        reportFirstConfirmedDTC = 0xC
        reportMostRecentTestFailedDTC = 0xD
        reportMostRecentConfirmedDTC = 0xE
        reportMirrorMemoryDTCByStatusMask = 0xF
        reportMirrorMemoryDTCExtendedDataRecordByDTCNumber = 0x10
        reportNumberOfMirrorMemoryDTCByStatusMask = 0x11
        reportNumberOfEmissionsRelatedOBDDTCByStatusMask = 0x12
        reportEmissionsRelatedOBDDTCByStatusMask = 0x13
        reportDTCFaultDetectionCounter = 0x14
        reportDTCWithPermanentStatus = 0x15

    @classmethod
    def assert_severity_mask(cls, severity_mask, subfunction):
        if severity_mask is None:
            raise ValueError('severity_mask must be provided for subfunction 0x%02x' % subfunction)
        ServiceHelper.validate_int(severity_mask, min=0, max=0xFF, name='Severity mask')

    @classmethod		
    def assert_status_mask(cls, status_mask, subfunction):
        if status_mask is None:
            raise ValueError('status_mask must be provided for subfunction 0x%02x' % subfunction)
        ServiceHelper.validate_int(status_mask, min=0, max=0xFF, name='Status mask')

    @classmethod	
    def assert_dtc(cls, dtc, subfunction):
        if dtc is None:
            raise ValueError('A dtc value must be provided for subfunction 0x%02x' % subfunction)
        ServiceHelper.validate_int(dtc, min=0, max=0xFFFFFF, name='DTC')

    @classmethod	
    def assert_snapshot_record_number(cls, snapshot_record_number, subfunction):
        if snapshot_record_number is None:
            raise ValueError('snapshot_record_number must be provided for subfunction 0x%02x' % subfunction)
        ServiceHelper.validate_int(snapshot_record_number, min=0, max=0xFF, name='Snapshot record number')

    @classmethod	
    def assert_extended_data_record_number(cls, extended_data_record_number, subfunction):
        if extended_data_record_number is None:
            raise ValueError('extended_data_record_number must be provided for subfunction 0x%02x' % subfunction)
        ServiceHelper.validate_int(extended_data_record_number, min=0, max=0xFF, name='Extended data record number')

    @classmethod	
    def assert_extended_data_size(cls, extended_data_size, subfunction):
        if extended_data_size is None:
            raise ValueError('extended_data_size must be provided as length of data is not given by the server.')
        ServiceHelper.validate_int(extended_data_size, min=0, max=0xFFF, name='Extended data size')

    @classmethod	
    def pack_dtc(cls, dtcid):
        return struct.pack('BBB', (dtcid >> 16) & 0xFF, (dtcid >> 8) & 0xFF,  (dtcid >> 0) & 0xFF)


    @classmethod
    def make_request(cls, subfunction, status_mask=None, severity_mask=None,  dtc=None, snapshot_record_number=None, extended_data_record_number=None):
        """
        Generates a request for ReadDTCInformation. 
        Each subfunction uses a subset of parameters. 

        :param subfunction: The service subfunction. Values are defined in :class:`ReadDTCInformation.Subfunction<ReadDTCInformation.Subfunction>`
        :type subfunction: int

        :param status_mask: A DTC status mask used to filter DTC
        :type status_mask: int or :ref:`Dtc.Status <DTC_Status>`

        :param severity_mask: A severity mask used to filter DTC 
        :type severity_mask: int or :ref:`Dtc.Severity <DTC_Severity>`

        :param dtc: A DTC mask used to filter DTC
        :type dtc: int or :ref:`Dtc <DTC>`

        :param snapshot_record_number: Snapshot record number
        :type snapshot_record_number: int

        :param extended_data_record_number: Extended data record number
        :type extended_data_record_number: int

        :raises ValueError: If parameters are out of range, missing or wrong type
        """	

        from udsoncan import Request, Dtc

        # Request grouping for subfunctions that have the same request format
        request_subfn_no_param = [
                ReadDTCInformation.Subfunction.reportSupportedDTCs,
                ReadDTCInformation.Subfunction.reportFirstTestFailedDTC,
                ReadDTCInformation.Subfunction.reportFirstConfirmedDTC,
                ReadDTCInformation.Subfunction.reportMostRecentTestFailedDTC,
                ReadDTCInformation.Subfunction.reportMostRecentConfirmedDTC,
                ReadDTCInformation.Subfunction.reportDTCFaultDetectionCounter,
                ReadDTCInformation.Subfunction.reportDTCWithPermanentStatus,

                # Documentation is confusing about reportDTCSnapshotIdentification subfunction.
                # It is presented with reportDTCSnapshotRecordByDTCNumber (2 params) but a footnote says that these 2 parameters
                # are not to be provided for reportDTCSnapshotIdentification. Therefore, it is the same as other no-params subfn
                ReadDTCInformation.Subfunction.reportDTCSnapshotIdentification	

                ]

        request_subfn_status_mask = [
                ReadDTCInformation.Subfunction.reportNumberOfDTCByStatusMask,
                ReadDTCInformation.Subfunction.reportDTCByStatusMask,
                ReadDTCInformation.Subfunction.reportMirrorMemoryDTCByStatusMask,
                ReadDTCInformation.Subfunction.reportNumberOfMirrorMemoryDTCByStatusMask,
                ReadDTCInformation.Subfunction.reportNumberOfEmissionsRelatedOBDDTCByStatusMask,
                ReadDTCInformation.Subfunction.reportEmissionsRelatedOBDDTCByStatusMask
        ]

        request_subfn_mask_record_plus_snapshot_record_number = [
                ReadDTCInformation.Subfunction.reportDTCSnapshotRecordByDTCNumber
        ]

        request_subfn_snapshot_record_number = [
                ReadDTCInformation.Subfunction.reportDTCSnapshotRecordByRecordNumber
        ]

        request_subfn_mask_record_plus_extdata_record_number = [
                ReadDTCInformation.Subfunction.reportDTCExtendedDataRecordByDTCNumber,
                ReadDTCInformation.Subfunction.reportMirrorMemoryDTCExtendedDataRecordByDTCNumber
        ]

        request_subfn_severity_plus_status_mask = [
                ReadDTCInformation.Subfunction.reportNumberOfDTCBySeverityMaskRecord,
                ReadDTCInformation.Subfunction.reportDTCBySeverityMaskRecord
        ]

        request_subfn_mask_record = [
                ReadDTCInformation.Subfunction.reportSeverityInformationOfDTC		
                ]


        ServiceHelper.validate_int(subfunction, min=1, max=0x15, name='Subfunction')

        if status_mask is not None and isinstance(status_mask, Dtc.Status):
            status_mask = status_mask.get_byte_as_int()

        if severity_mask is not None and isinstance(severity_mask, Dtc.Severity):
            severity_mask = severity_mask.get_byte_as_int()

        if dtc is not None and isinstance(dtc, Dtc):
            dtc = dtc.id

        req = Request(service=cls, subfunction=subfunction)

        if subfunction in request_subfn_no_param:		# Service ID + Subfunction
            pass

        elif subfunction in request_subfn_status_mask:
            cls.assert_status_mask(status_mask, subfunction)
            req.data = struct.pack('B', status_mask)

        elif subfunction in request_subfn_mask_record_plus_snapshot_record_number:
            cls.assert_dtc(dtc, subfunction)
            cls.assert_snapshot_record_number(snapshot_record_number, subfunction)
            req.data = cls.pack_dtc(dtc) + struct.pack('B', snapshot_record_number)

        elif subfunction in request_subfn_snapshot_record_number:
            cls.assert_snapshot_record_number(snapshot_record_number, subfunction)
            req.data = struct.pack('B', snapshot_record_number)

        elif subfunction in request_subfn_mask_record_plus_extdata_record_number:
            cls.assert_dtc(dtc, subfunction)
            cls.assert_extended_data_record_number(extended_data_record_number, subfunction)
            req.data = cls.pack_dtc(dtc) + struct.pack('B', extended_data_record_number)

        elif subfunction in request_subfn_severity_plus_status_mask:
            cls.assert_status_mask(status_mask, subfunction)
            cls.assert_severity_mask(severity_mask, subfunction)
            req.data = struct.pack('BB', severity_mask, status_mask)

        elif subfunction in request_subfn_mask_record:
            cls.assert_dtc(dtc, subfunction)
            req.data = cls.pack_dtc(dtc)

        return req

    @classmethod
    def interpret_response(cls, response, subfunction,  extended_data_size=None, tolerate_zero_padding=True, ignore_all_zero_dtc=True, dtc_snapshot_did_size=2, didconfig=None):
        """
        Populates the response ``service_data`` property with an instance of :class:`ReadDTCInformation.ResponseData<udsoncan.services.ReadDTCInformation.ResponseData>`

        :param response: The received response to interpret
        :type response: :ref:`Response<Response>`

        :param subfunction: The service subfunction. Values are defined in :class:`ReadDTCInformation.Subfunction<udsoncan.services.ReadDTCInformation.Subfunction>`
        :type subfunction: int

        :param extended_data_size: Extended data size to expect. Extended data is implementation specific, therefore, size is not standardized
        :type extended_data_size: int

        :param tolerate_zero_padding: Ignore trailing zeros in the response data avoiding raising false :class:`InvalidResponseException<udsoncan.exceptions.InvalidResponseException>`.
        :type tolerate_zero_padding:  bool

        :param ignore_all_zero_dtc: Discard any DTC entries that have an ID of 0. Avoid reading extra DTCs when using a transport protocol using zero padding.
        :type ignore_all_zero_dtc: bool

        :param dtc_snapshot_did_size: Number of bytes to encode the data identifier number. Other services such as :ref:`ReadDataByIdentifier<ReadDataByIdentifier>` encode DID over 2 bytes.
                UDS standard does not define the size of the snapshot DID, therefore, it must be supplied.
        :type dtc_snapshot_did_size: int

        :param didconfig: Definition of DID codecs. Dictionary mapping a DID (int) to a valid :ref:`DidCodec<DidCodec>` class or pack/unpack string 
        :type didconfig: dict[int] = :ref:`DidCodec<DidCodec>`

        :raises InvalidResponseException: If response length is wrong or does not match DID configuration
        :raises ValueError: If parameters are out of range, missing or wrong types
        :raises ConfigError: If the server returns a snapshot DID not defined in ``didconfig``
        """	

        from udsoncan import Dtc, DidCodec
        ServiceHelper.validate_int(subfunction, min=1, max=0x15, name='Subfunction')

        # Response grouping for responses that are encoded the same way
        response_subfn_dtc_availability_mask_plus_dtc_record = [
                ReadDTCInformation.Subfunction.reportDTCByStatusMask,
                ReadDTCInformation.Subfunction.reportSupportedDTCs,
                ReadDTCInformation.Subfunction.reportFirstTestFailedDTC,
                ReadDTCInformation.Subfunction.reportFirstConfirmedDTC,
                ReadDTCInformation.Subfunction.reportMostRecentTestFailedDTC,
                ReadDTCInformation.Subfunction.reportMostRecentConfirmedDTC,
                ReadDTCInformation.Subfunction.reportMirrorMemoryDTCByStatusMask,
                ReadDTCInformation.Subfunction.reportEmissionsRelatedOBDDTCByStatusMask,
                ReadDTCInformation.Subfunction.reportDTCWithPermanentStatus
        ]

        response_subfn_number_of_dtc = [
                ReadDTCInformation.Subfunction.reportNumberOfDTCByStatusMask,
                ReadDTCInformation.Subfunction.reportNumberOfDTCBySeverityMaskRecord,
                ReadDTCInformation.Subfunction.reportNumberOfMirrorMemoryDTCByStatusMask,
                ReadDTCInformation.Subfunction.reportNumberOfEmissionsRelatedOBDDTCByStatusMask,
        ]

        response_subfn_dtc_availability_mask_plus_dtc_record_with_severity = [
                ReadDTCInformation.Subfunction.reportDTCBySeverityMaskRecord,
                ReadDTCInformation.Subfunction.reportSeverityInformationOfDTC
        ]

        response_subfn_dtc_plus_fault_counter = [
                ReadDTCInformation.Subfunction.reportDTCFaultDetectionCounter
        ]

        response_subfn_dtc_plus_sapshot_record = [
                ReadDTCInformation.Subfunction.reportDTCSnapshotIdentification
        ]

        response_sbfn_dtc_status_snapshots_records = [
                ReadDTCInformation.Subfunction.reportDTCSnapshotRecordByDTCNumber
        ]

        response_sbfn_dtc_status_snapshots_records_record_first = [
                ReadDTCInformation.Subfunction.reportDTCSnapshotRecordByRecordNumber
        ]

        response_subfn_mask_record_plus_extdata = [
                ReadDTCInformation.Subfunction.reportDTCExtendedDataRecordByDTCNumber,
                ReadDTCInformation.Subfunction.reportMirrorMemoryDTCExtendedDataRecordByDTCNumber
        ]


        response.service_data = cls.ResponseData()	# what will be returned 

        if len(response.data) < 1:
            raise InvalidResponseException(response, 'Response must be at least 1 byte long (echo of subfunction)')

        response.service_data.subfunction_echo = response.data[0]	# First byte is subfunction

        # Now for each response group, we have a different decoding algorithm
        if subfunction in response_subfn_dtc_availability_mask_plus_dtc_record + response_subfn_dtc_availability_mask_plus_dtc_record_with_severity:

            if subfunction in response_subfn_dtc_availability_mask_plus_dtc_record:
                dtc_size = 4	# DTC ID (3) + Status (1)
            elif subfunction in response_subfn_dtc_availability_mask_plus_dtc_record_with_severity:
                dtc_size = 6	# DTC ID (3) + Status (1) + Severity (1) + FunctionalUnit (1)

            if len(response.data) < 2:
                raise InvalidResponseException(response, 'Response must be at least 2 byte long (echo of subfunction and DTCStatusAvailabilityMask)')

            response.service_data.status_availability = Dtc.Status.from_byte(response.data[1])

            actual_byte = 2	# Increasing index
            while True:	# Loop until we have read all dtcs
                if len(response.data) <= actual_byte:
                    break	# done

                elif len(response.data) < actual_byte+dtc_size:
                    partial_dtc_length = len(response.data)-actual_byte
                    if tolerate_zero_padding and response.data[actual_byte:] == b'\x00'*partial_dtc_length:
                        break
                    else:
                        # We purposely ignore extra byte for subfunction reportSeverityInformationOfDTC as it is supposed to return 0 or 1 DTC.
                        if subfunction != ReadDTCInformation.Subfunction.reportSeverityInformationOfDTC or actual_byte == 2: 
                            raise InvalidResponseException(response, 'Incomplete DTC record. Missing %d bytes to response to complete the record' % (dtc_size-partial_dtc_length))

                else:
                    dtc_bytes = response.data[actual_byte:actual_byte+dtc_size]
                    if dtc_bytes == b'\x00'*dtc_size and ignore_all_zero_dtc:
                        pass # ignore
                    else:
                        if subfunction in response_subfn_dtc_availability_mask_plus_dtc_record:
                            dtc = Dtc(struct.unpack('>L', b'\x00' + dtc_bytes[0:3])[0])
                            dtc.status.set_byte(dtc_bytes[3])
                        elif subfunction in response_subfn_dtc_availability_mask_plus_dtc_record_with_severity:
                            dtc = Dtc(struct.unpack('>L', b'\x00' + dtc_bytes[2:5])[0])
                            dtc.severity.set_byte(dtc_bytes[0])
                            dtc.functional_unit = dtc_bytes[1]
                            dtc.status.set_byte(dtc_bytes[5])

                        response.service_data.dtcs.append(dtc)
                actual_byte += dtc_size
            response.service_data.dtc_count = len(response.service_data.dtcs)

        # The 2 following subfunction responses have different purposes but their constructions are very similar.
        elif subfunction in response_subfn_dtc_plus_fault_counter + response_subfn_dtc_plus_sapshot_record:
            dtc_size = 4
            if len(response.data) < 1:
                raise InvalidResponseException(response, 'Response must be at least 1 byte long (echo of subfunction)')

            actual_byte = 1 	# Increasing index
            dtc_map = dict()	# This map is used to append snapshot to existing DTC.

            while True: 	# Loop until we have read all dtcs
                if len(response.data) <= actual_byte:
                    break 	# done

                elif len(response.data) < actual_byte+dtc_size:
                    partial_dtc_length = len(response.data)-actual_byte
                    if tolerate_zero_padding and response.data[actual_byte:] == b'\x00'*partial_dtc_length:
                        break
                    else:
                        raise InvalidResponseException(response, 'Incomplete DTC record. Missing %d bytes to response to complete the record' % (dtc_size-partial_dtc_length))
                else:
                    dtc_bytes = response.data[actual_byte:actual_byte+dtc_size]
                    if dtc_bytes == b'\x00'*dtc_size and ignore_all_zero_dtc:
                        pass # ignore
                    else:		
                        dtcid = struct.unpack('>L', b'\x00' + dtc_bytes[0:3])[0]
                        # We create the DTC or get its reference if already created.
                        dtc_created = False	
                        if dtcid in dtc_map and subfunction in response_subfn_dtc_plus_sapshot_record:
                            dtc = dtc_map[dtcid]
                        else:
                            dtc = Dtc(dtcid)
                            dtc_map[dtcid] = dtc
                            dtc_created = True

                        # We either read the DTC fault counter or Snapshot record number. 
                        if subfunction in response_subfn_dtc_plus_fault_counter:
                            dtc.fault_counter = dtc_bytes[3]

                        elif subfunction in response_subfn_dtc_plus_sapshot_record:
                            record_number = dtc_bytes[3]

                            if dtc.snapshots is None:
                                dtc.snapshots = []

                            dtc.snapshots.append(record_number)

                        # Adds the DTC to the list.
                        if dtc_created:
                            response.service_data.dtcs.append(dtc)

                actual_byte += dtc_size

            response.service_data.dtc_count = len(response.service_data.dtcs)

        # This group of responses returns a number of DTCs available
        elif subfunction in response_subfn_number_of_dtc:
            if len(response.data) < 5:
                raise InvalidResponseException(response, 'Response must be exactly 5 bytes long ')

            response.service_data.status_availability = Dtc.Status.from_byte(response.data[1])
            response.service_data.dtc_format = response.data[2]
            response.service_data.dtc_count = struct.unpack('>H', response.data[3:5])[0]

        # This group of responses returns DTC snapshots
        # Responses include a DTC, many snapshot records. For each record, we find many Data Identifiers.
        # We create one Dtc.Snapshot for each DID. That'll be easier to work with.
        # <DTC,RecordNumber1,NumberOfDid_X,DID1,DID2,...DIDX, RecordNumber2,NumberOfDid_Y,DID1,DID2,...DIDY, etc>
        elif subfunction in  response_sbfn_dtc_status_snapshots_records:
            if len(response.data) < 5:
                raise InvalidResponseException(response, 'Response must be at least 5 bytes long ')

            dtc = Dtc(struct.unpack('>L', b'\x00' + response.data[1:4])[0])
            dtc.status.set_byte(response.data[4])
            actual_byte = 5		# Increasing index

            ServiceHelper.validate_int(dtc_snapshot_did_size, min=1, max=8, name='dtc_snapshot_did_size')

            while True:		# Loop until we have read all dtcs
                if len(response.data) <= actual_byte:
                    break	# done

                remaining_data = response.data[actual_byte:]
                if tolerate_zero_padding and remaining_data == b'\x00' * len(remaining_data):
                    break

                if len(remaining_data) < 2:
                    raise InvalidResponseException(response, 'Incomplete response from server. Missing "number of identifier" and following data')

                record_number = remaining_data[0]	
                number_of_did = remaining_data[1]
                # Validate record number and number of DID before continuing
                if number_of_did == 0:
                    raise InvalidResponseException(response, 'Server returned snapshot record #%d with no data identifier included' % (record_number)) 

                if len(remaining_data) < 2 + dtc_snapshot_did_size:
                    raise InvalidResponseException(response, 'Incomplete response from server. Missing DID number and associated data.')

                actual_byte += 2
                for i in range(number_of_did):
                    remaining_data = response.data[actual_byte:]
                    snapshot = Dtc.Snapshot()	# One snapshot per DID for convenience.
                    snapshot.record_number = record_number

                    # As standard does not specify the length of the DID, we craft it based on a config 
                    did = 0
                    for j in range(dtc_snapshot_did_size):
                        offset = dtc_snapshot_did_size-1-j
                        did |= (remaining_data[offset] << (8*j))

                    # Decode the data based on DID number.
                    snapshot.did = did
                    didconfig = ServiceHelper.check_did_config(did, didconfig)
                    codec = DidCodec.from_config(didconfig[did])

                    data_offset =  dtc_snapshot_did_size;
                    if len(remaining_data[data_offset:]) < len(codec):
                        raise InvalidResponseException(response, 'Incomplete response. Data for DID 0x%04x is only %d bytes while %d bytes is expected' % (did, len(remaining_data[data_offset:]), len(codec)))

                    snapshot.raw_data = remaining_data[data_offset:data_offset + len(codec)]
                    snapshot.data = codec.decode(snapshot.raw_data)

                    dtc.snapshots.append(snapshot)
                    actual_byte += dtc_snapshot_did_size + len(codec)

            response.service_data.dtcs.append(dtc)
            response.service_data.dtc_count = 1

        # This group of responses returns DTC snapshots
        # Responses include a DTC, many snapshots records. For each record, we find many Data Identifiers.
        # We create one Dtc.Snapshot for each DID. That'll be easier to work with.
        # Similar to previous subfunction group, but order of information is changed.

        # <RecordNumber1, DTC1,NumberOfDid_X,DID1,DID2,...DIDX, RecordNumber2,DTC2, NumberOfDid_Y,DID1,DID2,...DIDY, etc>
        elif subfunction in response_sbfn_dtc_status_snapshots_records_record_first :
            ServiceHelper.validate_int(dtc_snapshot_did_size, min=1, max=8, name='dtc_snapshot_did_size')

            if len(response.data) < 2:
                raise InvalidResponseException(response, 'Response must be at least 2 bytes long. Subfunction echo + RecordNumber ')

            actual_byte = 1	 # Increasing index
            while True:	# Loop through response data
                if len(response.data) <= actual_byte:
                    break	# done

                remaining_data = response.data[actual_byte:]
                record_number = remaining_data[0]

                # If empty response but filled with 0, it is considered ok
                if remaining_data == b'\x00' * len(remaining_data) and tolerate_zero_padding:
                    break

                # If record number received but no DTC provided (allowed according to standard), we exit.
                if len(remaining_data) == 1 or tolerate_zero_padding and remaining_data[1:] == b'\x00' * len(remaining_data[1:]):
                    break

                if len(remaining_data) < 5: 	# Partial DTC (No DTC at all is checked above)
                    raise InvalidResponseException(response, 'Incomplete response from server. Missing "DTCAndStatusRecord" and following data')

                if len(remaining_data) < 6:
                    raise InvalidResponseException(response, 'Incomplete response from server. Missing number of data identifier')

                # DTC decoding
                dtc = Dtc(struct.unpack('>L', b'\x00' + remaining_data[1:4])[0])
                dtc.status.set_byte(remaining_data[4])
                number_of_did = remaining_data[5]

                actual_byte += 6
                remaining_data = response.data[actual_byte:]

                if number_of_did == 0:
                    raise InvalidResponseException(response, 'Server returned snapshot record #%d with no data identifier included' % (record_number)) 

                if len(remaining_data) < dtc_snapshot_did_size:
                    raise InvalidResponseException(response, 'Incomplete response from server. Missing DID and associated data')

                # We have a DTC and 0 DID, next loop
                if tolerate_zero_padding and remaining_data == b'\x00' * len(remaining_data):
                    break

                # For each DID
                for i in range(number_of_did):
                    remaining_data = response.data[actual_byte:]
                    snapshot = Dtc.Snapshot()	# One snapshot epr DID for convenience
                    snapshot.record_number = record_number

                    # As standard does not specify the length of the DID, we craft it based on a config 
                    did = 0
                    for j in range(dtc_snapshot_did_size):
                        offset = dtc_snapshot_did_size-1-j
                        did |= (remaining_data[offset] << (8*j))

                    # Decode the data based on DID number.
                    snapshot.did = did
                    didconfig = ServiceHelper.check_did_config(did, didconfig)
                    codec = DidCodec.from_config(didconfig[did])

                    data_offset =  dtc_snapshot_did_size;
                    if len(remaining_data[data_offset:]) < len(codec):
                        raise InvalidResponseException(response, 'Incomplete response. Data for DID 0x%04x is only %d bytes while %d bytes is expected' % (did, len(remaining_data[data_offset:]), len(codec)))

                    snapshot.raw_data = remaining_data[data_offset:data_offset + len(codec)]
                    snapshot.data = codec.decode(snapshot.raw_data)

                    dtc.snapshots.append(snapshot)

                    actual_byte += dtc_snapshot_did_size + len(codec)

                response.service_data.dtcs.append(dtc)
            response.service_data.dtc_count = len(response.service_data.dtcs)

        # These subfunctions include DTC ExtraData. We give it raw to user.
        elif subfunction in response_subfn_mask_record_plus_extdata:
            cls.assert_extended_data_size(extended_data_size, subfunction)

            if len(response.data) < 5: 
                raise InvalidResponseException(response, 'Incomplete response from server. Missing DTCAndStatusRecord')
            # DTC decoding
            dtc = Dtc(struct.unpack('>L', b'\x00' + response.data[1:4])[0])
            dtc.status.set_byte(response.data[4])

            actual_byte = 5	# Increasing index
            while actual_byte < len(response.data):	# Loop through data
                remaining_data = response.data[actual_byte:]
                record_number = remaining_data[0]

                if record_number == 0:
                    if remaining_data == b'\x00' * len(remaining_data) and tolerate_zero_padding:
                        break
                    else:
                        raise InvalidResponseException(response, 'Extended data record number given by the server is 0 but this value is a reserved value.')

                actual_byte +=1
                remaining_data = response.data[actual_byte:]

                if len(remaining_data) < extended_data_size:
                    raise InvalidResponseException(response, 'Incomplete response from server. Length of extended data for DTC 0x%06x with record number 0x%02x is %d bytes but smaller than given data_size of %d bytes' % (dtc.id, record_number, len(remaining_data), extended_data_size))

                exdata = Dtc.ExtendedData()
                exdata.record_number = record_number
                exdata.raw_data = remaining_data[0:extended_data_size]

                dtc.extended_data.append(exdata)

                actual_byte+= extended_data_size

            response.service_data.dtcs.append(dtc)
            response.service_data.dtc_count = len(response.service_data.dtcs)

    class ResponseData(BaseResponseData):
        """
        .. data:: subfunction_echo

                Subfunction echoed back by the server

        .. data:: dtcs

                :ref:`DTC<DTC>` instances and their status read from the server.

        .. data:: dtc_count

                Number of DTC read or available

        .. data:: dtc_format

                Integer indicating the format of the DTC. See :ref:`DTC.Format<DTC_Format>`

        .. data:: status_availability

                :ref:`Dtc.Status<DTC_Status>` indicating which status the server supports

        .. data:: extended_data

                List of bytes containing the DTC extended data

        """			
        def __init__(self):
            super().__init__(ReadDTCInformation)
            self.subfunction_echo = None
            self.dtcs = []
            self.dtc_count = 0
            self.dtc_format = None
            self.status_availability = None
            self.extended_data = []	



from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *

class TesterPresent(BaseService):
    _sid = 0x3E

    supported_negative_response = [	Response.Code.SubFunctionNotSupported, 
                                                    Response.Code.IncorrectMessageLegthOrInvalidFormat
                                                    ]	

    @classmethod
    def make_request(cls):
        """
        Generates a request for TesterPresent
        """		
        from udsoncan import Request
        return Request(service=cls, subfunction=0)

    @classmethod
    def interpret_response(cls, response):
        """
        Populates the response ``service_data`` property with an instance of :class:`TesterPresent.ResponseData<udsoncan.services.TesterPresent.ResponseData>`

        :param response: The received response to interpret
        :type response: :ref:`Response<Response>`

        :raises InvalidResponseException: If length of ``response.data`` is too short
        """		
        if  len(response.data) < 1:
            raise InvalidResponseException(response, "Response data must be at least 1 bytes")

        response.service_data = cls.ResponseData()
        response.service_data.subfunction_echo = response.data[0]

    class ResponseData(BaseResponseData):
        """
        .. data:: subfunction_echo

                Requests subfunction echoed back by the server. This value should always be 0
        """		
        def __init__(self):
            super().__init__(TesterPresent)
            self.subfunction_echo = None



from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *

class CommunicationControl(BaseService):
    _sid = 0x28

    class ControlType(BaseSubfunction):
        """
        CommunicationControl defined subfunctions
        """		
        __pretty_name__ = 'control type' 

        enableRxAndTx = 0
        enableRxAndDisableTx = 1
        disableRxAndEnableTx = 2
        disableRxAndTx = 3

    supported_negative_response = [	Response.Code.SubFunctionNotSupported, 
                                                    Response.Code.IncorrectMessageLegthOrInvalidFormat,
                                                    Response.Code.ConditionsNotCorrect,
                                                    Response.Code.RequestOutOfRange
                                                    ]

    @classmethod
    def normalize_communication_type(self, communication_type):
        from udsoncan import CommunicationType

        if not isinstance(communication_type, CommunicationType) and not isinstance(communication_type, int) and not isinstance(communication_type, bytes):
            raise ValueError('communication_type must either be a CommunicationType object or an integer')

        if isinstance(communication_type, int) or isinstance(communication_type, bytes):
            communication_type = CommunicationType.from_byte(communication_type)

        return communication_type

    @classmethod
    def make_request(cls, control_type, communication_type):
        """
        Generates a request for CommunicationControl

        :param control_type: Service subfunction. Allowed values are from 0 to 0x7F
        :type control_type: int

        :param communication_type: The communication type requested.
        :type communication_type: :ref:`CommunicationType <CommunicationType>`, int, bytes

        :raises ValueError: If parameters are out of range, missing or wrong type
        """		
        from udsoncan import Request

        ServiceHelper.validate_int(control_type, min=0, max=0x7F, name='Control type')

        communication_type = cls.normalize_communication_type(communication_type)
        request = Request(service=cls, subfunction=control_type)
        request.data = communication_type.get_byte()

        return request

    @classmethod
    def interpret_response(cls, response):
        """
        Populates the response ``service_data`` property with an instance of :class:`CommunicationControl.ResponseData<udsoncan.services.CommunicationControl.ResponseData>`

        :param response: The received response to interpret
        :type response: :ref:`Response<Response>`

        :raises InvalidResponseException: If length of ``response.data`` is too short
        """		
        if len(response.data) < 1: 	
            raise InvalidResponseException(response, "Response data must be at least 1 byte")

        response.service_data = cls.ResponseData()
        response.service_data.control_type_echo = response.data[0]

    class ResponseData(BaseResponseData):
        """
        .. data:: control_type_echo

                Request subfunction echoed back by the server
        """		
        def __init__(self):
            super().__init__(CommunicationControl)
            self.control_type_echo = None



import udsoncan
from udsoncan.connections import IsoTPConnection, PythonIsoTpConnection
from udsoncan.client import Client
from udsoncan.exceptions import *
from udsoncan.services import *
import udsoncan.configs
import can


def algo(seed, param):
    return b"\xed\xcb\xa9\x87"


class my_codec(udsoncan.DidCodec):
    def encode(self, s):
        return bytes(s.encode('ascii'))


config = dict(udsoncan.configs.default_client_config)
config['data_identifiers'] = {0xF190: my_codec, 1: 'H', 2: 'H', 3: 'i'}
config['security_algo'] = algo

conn = IsoTPConnection('vcan0', rxid=0x123, txid=0x456)
#bus = can.interface.Bus('vcan0', bustype='socketcan')
#conn = PythonIsoTpConnection(bus, txid=0x456, rxid=0x123, params={'stmin' : 1, })
with Client(conn,  request_timeout=2, config=config) as client:
    try:
        # integer with value of 3
        client.change_session(
            DiagnosticSessionControl.Session.extendedDiagnosticSession)
        # Fictive security level. Integer coming from fictive lib, let's say its value is 5
        client.unlock_security_access(3)
        # Standard ID for VIN is 0xF190. Codec is set in the client configuration
        response = client.read_data_by_identifier([1, 2, 3])
        print(response)
        client.ecu_reset(ECUReset.ResetType.hardReset)  # HardReset = 0x01
    except NegativeResponseException as e:
        print('Server refused our request for service %s with code "%s" (0x%02x)' % (
            e.response.service.get_name(), e.response.code_name, e.response.code))
    except InvalidResponseException as e:
        print('Server sent an invalid payload : %s' %
              e.response.original_payload)



from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *

class ReadMemoryByAddress(BaseService):
    _sid = 0x23
    _use_subfunction = False

    supported_negative_response = [	 Response.Code.IncorrectMessageLegthOrInvalidFormat,
                                                    Response.Code.ConditionsNotCorrect,
                                                    Response.Code.RequestOutOfRange,
                                                    Response.Code.SecurityAccessDenied
                                                    ]

    @classmethod
    def make_request(cls, memory_location):
        """
        Generates a request for ReadMemoryByAddress

        :param memory_location: The address and the size of the memory block to read.
        :type memory_location: :ref:`MemoryLocation <MemoryLocation>`

        :raises ValueError: If parameters are out of range, missing or wrong type
        """		
        from udsoncan import Request, MemoryLocation

        if not isinstance(memory_location, MemoryLocation):
            raise ValueError('Given memory location must be an instance of MemoryLocation')

        request =  Request(service=cls)
        request.data = b''
        request.data += memory_location.alfid.get_byte() # AddressAndLengthFormatIdentifier
        request.data += memory_location.get_address_bytes()
        request.data += memory_location.get_memorysize_bytes()

        return request

    @classmethod
    def interpret_response(cls, response):
        """
        Populates the response ``service_data`` property with an instance of :class:`ReadMemoryByAddress.ResponseData<udsoncan.services.ReadMemoryByAddress.ResponseData>`

        :param response: The received response to interpret
        :type response: :ref:`Response<Response>`

        :raises InvalidResponseException: If length of ``response.data`` is too short
        """		
        if len(response.data) < 1: 	
            raise InvalidResponseException(response, "Response data must be at least 1 byte")

        response.service_data = cls.ResponseData()
        response.service_data.memory_block = response.data

    class ResponseData(BaseResponseData):
        """
        .. data:: memory_block

                bytes object reflecting the content of the read memory
        """
        def __init__(self):
            super().__init__(ReadMemoryByAddress)
            self.memory_block = None



from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *

class ReadScalingDataByIdentifier(BaseService):
    _sid = 0x24

    supported_negative_response = [	 Response.Code.IncorrectMessageLegthOrInvalidFormat,
                                                    Response.Code.ConditionsNotCorrect,
                                                    Response.Code.RequestOutOfRange,
                                                    Response.Code.SecurityAccessDenied
                                                    ]

    @classmethod
    def make_request(cls):
        raise NotImplementedError('Service is not implemented')

    @classmethod
    def interpret_response(cls, response):
        raise NotImplementedError('Service is not implemented')

    class ResponseData(BaseResponseData):	
        def __init__(self):
            super().__init__(ReadScalingDataByIdentifier)



from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *
import struct


class ReadDataByIdentifier(BaseService):
    _sid = 0x22
    _use_subfunction = False


    supported_negative_response = [	 Response.Code.IncorrectMessageLegthOrInvalidFormat,
                                                    Response.Code.ConditionsNotCorrect,
                                                    Response.Code.RequestOutOfRange,
                                                    Response.Code.SecurityAccessDenied
                                                    ]	

    @classmethod
    def validate_didlist_input(cls, dids):
        if not isinstance(dids, int) and not isinstance(dids, list):
            raise ValueError("Data Identifier must either be an integer or a list of integer")

        if isinstance(dids, int):
            ServiceHelper.validate_int(dids, min=0, max=0xFFFF, name='Data Identifier')

        if isinstance(dids, list):
            for did in dids:
                ServiceHelper.validate_int(did, min=0, max=0xFFFF, name='Data Identifier')


        return [dids] if not isinstance(dids, list) else dids

    @classmethod
    def make_request(cls, didlist, didconfig):
        """
        Generates a request for ReadDataByIdentifier

        :param didlist: List of data identifier to read.
        :type didlist: list[int]

        :param didconfig: Definition of DID codecs. Dictionary mapping a DID (int) to a valid :ref:`DidCodec<DidCodec>` class or pack/unpack string 
        :type didconfig: dict[int] = :ref:`DidCodec<DidCodec>`

        :raises ValueError: If parameters are out of range, missing or wrong type
        :raises ConfigError: If didlist contains a DID not defined in didconfig
        """		
        from udsoncan import Request
        from udsoncan import DidCodec

        didlist = cls.validate_didlist_input(didlist)

        req = Request(cls)
        ServiceHelper.check_did_config(didlist, didconfig)

        did_reading_all_data = None
        for did in didlist:
            if did not in didconfig:    # Already checked in check_did_config. Paranoid check
                raise ConfigError(key=did, msg='Actual data identifier configuration contains no definition for data identifier 0x%04x' % did)
                
            codec = DidCodec.from_config(didconfig[did])
            try:
                length = len(codec)
                if did_reading_all_data is not None:
                    raise ValueError('Did 0x%04X is configured to read the rest of the payload (__len__ raisong ReadAllRemainingData), but a subsequent DID is requested (0x%04x)' % (did_reading_all_data, did))
            except DidCodec.ReadAllRemainingData:
                if did_reading_all_data is not None:
                    raise ValueError('It is impossible to read 2 DIDs configured to read the rest of the payload (__len__ raising ReadAllRemainingData). Dids are : 0x%04X and 0x%04X' % (did_reading_all_data, did))
                did_reading_all_data = did

        req.data = struct.pack('>'+'H'*len(didlist), *didlist) #Encode list of DID

        return req

    @classmethod
    def interpret_response(cls, response, didlist, didconfig, tolerate_zero_padding=True):
        """
        Populates the response ``service_data`` property with an instance of :class:`ReadDataByIdentifier.ResponseData<udsoncan.services.ReadDataByIdentifier.ResponseData>`

        :param response: The received response to interpret
        :type response: :ref:`Response<Response>`

        :param didlist:  List of data identifiers used for the request.
        :type didlist: list[int]

        :param didconfig: Definition of DID codecs. Dictionary mapping a DID (int) to a valid :ref:`DidCodec<DidCodec>` class or pack/unpack string 
        :type didconfig: dict[int] = :ref:`DidCodec<DidCodec>`

        :param tolerate_zero_padding: Ignore trailing zeros in the response data avoiding raising false :class:`InvalidResponseException<udsoncan.exceptions.InvalidResponseException>`.
        :type tolerate_zero_padding: bool

        :raises ValueError: If parameters are out of range, missing or wrong type
        :raises ConfigError: If ``didlist`` parameter or response contains a DID not defined in ``didconfig``.
        :raises InvalidResponseException: If response data is incomplete or if DID data does not match codec length.
        """	
        from udsoncan import DidCodec

        didlist = cls.validate_didlist_input(didlist)
        didconfig = ServiceHelper.check_did_config(didlist, didconfig)

        response.service_data = cls.ResponseData()
        response.service_data.values = {}

        # Parsing algorithm to extract DID value
        offset = 0
        while True:
            if len(response.data) <= offset:
                break	# Done

            if len(response.data) <= offset +1:
                if tolerate_zero_padding and response.data[-1] == 0:	# One extra byte, but it's a 0 and we accept that. So we're done
                    break
                raise InvalidResponseException(response, "Response given by server is incomplete.")

            did = struct.unpack('>H', response.data[offset:offset+2])[0]	# Get the DID number
            if did == 0 and did not in didconfig and tolerate_zero_padding: # We read two zeros and that is not a DID bu we accept that. So we're done.
                if response.data[offset:] == b'\x00' * (len(response.data) - offset):
                    break

            if did not in didconfig:	# Already checked in check_did_config. Paranoid check
                raise ConfigError(key=did, msg='Actual data identifier configuration contains no definition for data identifier 0x%04x' % did)

            codec = DidCodec.from_config(didconfig[did])
            offset+=2

            try:
                payload_size = len(codec)
            except DidCodec.ReadAllRemainingData:
                payload_size = len(response.data) - offset

            if len(response.data) < offset+payload_size:
                raise InvalidResponseException(response, "Value for data identifier 0x%04x was incomplete according to definition in configuration" % did)

            subpayload = response.data[offset:offset+payload_size]
            offset += payload_size	# Codec must define a __len__ function that matches the encoded payload length.
            val = codec.decode(subpayload)
            response.service_data.values[did] = val

        return response

    class ResponseData(BaseResponseData):
        """
        .. data:: values

                Dictionary mapping the DID (int) with the value returned by the associated :ref:`DidCodec<DidCodec>`.decode method
        """				
        def __init__(self):
            super().__init__(ReadDataByIdentifier)

            self.values = None



from udsoncan import services
from udsoncan.exceptions import *
from udsoncan import Request, Response

from test.ClientServerTest import ClientServerTest
import time

class TestClient(ClientServerTest):
    def __init__(self, *args, **kwargs):
        ClientServerTest.__init__(self, *args, **kwargs)

    def test_timeout_override(self):
        pass

    def _test_timeout_override(self):
        req = Request(service = services.TesterPresent, subfunction=0) 
        timeout = 0.5
        try:
            t1 = time.time()
            response = self.udsclient.send_request(req, timeout=timeout)
            raise Exception('Request did not raise a TimeoutException')
        except TimeoutException as e:
            diff = time.time() - t1
            self.assertGreater(diff, timeout, 'Timeout raised after %.3f seconds when it should be %.3f sec' % (diff, timeout))
            self.assertLess(diff, timeout+0.5, 'Timeout raised after %.3f seconds when it should be %.3f sec' % (diff, timeout))

    # Server does not respond. Overall timeout is set smaller than P2 timeout. Overall timeout should trig
    def test_no_response_overall_timeout(self):
        pass

    def _test_no_response_overall_timeout(self):
        req = Request(service = services.TesterPresent, subfunction=0) 
        timeout = 0.5
        self.udsclient.set_configs({'request_timeout': timeout, 'p2_timeout' : 2, 'p2_star_timeout':2})
        try:
            t1 = time.time()
            response = self.udsclient.send_request(req)
            raise Exception('Request did not raise a TimeoutException')
        except TimeoutException as e:
            diff = time.time() - t1
            self.assertGreater(diff, timeout, 'Timeout raised after %.3f seconds when it should be %.3f sec' % (diff, timeout))
            self.assertLess(diff, timeout+0.5, 'Timeout raised after %.3f seconds when it should be %.3f sec' % (diff, timeout))

    # Server does not respond. P2 timeout is smaller than overall timeout. P2 timeout should trig
    def test_no_response_p2_timeout(self):
        pass

    def _test_no_response_p2_timeout(self):
        req = Request(service = services.TesterPresent, subfunction=0) 
        timeout = 0.5
        self.udsclient.set_configs({'request_timeout': 2, 'p2_timeout' : timeout, 'p2_star_timeout':2})
        try:
            t1 = time.time()
            response = self.udsclient.send_request(req)
            raise Exception('Request did not raise a TimeoutException')
        except TimeoutException as e:
            diff = time.time() - t1
            self.assertGreater(diff, timeout, 'Timeout raised after %.3f seconds when it should be %.3f sec' % (diff, timeout))
            self.assertLess(diff, timeout+0.5, 'Timeout raised after %.3f seconds when it should be %.3f sec' % (diff, timeout))

    #  overall timeout is set to 0.5. Server respond "pendingResponse" for 1 sec. Client should timeout first.
    def test_overall_timeout_pending_response(self):
        if not hasattr(self, 'completed'):
            self.completed = False
        self.conn.touserqueue.get(timeout=0.2)
        response = Response(service=services.TesterPresent, code=Response.Code.RequestCorrectlyReceived_ResponsePending)
        t1 = time.time()
        while time.time() - t1 < 1 and not self.completed:
            self.conn.fromuserqueue.put(response.get_payload())
            time.sleep(0.1)

    def _test_overall_timeout_pending_response(self):
        req = Request(service = services.TesterPresent, subfunction=0) 
        timeout = 0.5
        self.udsclient.set_configs({'request_timeout': timeout, 'p2_timeout' : 2, 'p2_star_timeout':2})
        try:
            t1 = time.time()
            response = self.udsclient.send_request(req)
            raise Exception('Request did not raise a TimeoutException')
        except TimeoutException as e:
            self.assertIsNotNone(self.udsclient.last_response, 'Client never received the PendingResponse message')
            self.completed = True
            diff = time.time() - t1
            self.assertGreater(diff, timeout, 'Timeout raised after %.3f seconds when it should be %.3f sec' % (diff, timeout))
            self.assertLess(diff, timeout+0.5, 'Timeout raised after %.3f seconds when it should be %.3f sec' % (diff, timeout))

    #  Sends 2 "pending response" response to switch to P2* timeout.
    def test_p2_star_timeout(self):
        self.conn.touserqueue.get(timeout=0.2)
        response = Response(service=services.TesterPresent, code=Response.Code.RequestCorrectlyReceived_ResponsePending)
        self.conn.fromuserqueue.put(response.get_payload())
        time.sleep(0.1)
        self.conn.fromuserqueue.put(response.get_payload())

    def _test_p2_star_timeout(self):
        req = Request(service = services.TesterPresent, subfunction=0) 
        timeout = 2
        self.udsclient.set_configs({'request_timeout': 5, 'p2_timeout' : 0.5, 'p2_star_timeout':timeout})
        try:
            t1 = time.time()
            response = self.udsclient.send_request(req)
            raise Exception('Request did not raise a TimeoutException')
        except TimeoutException as e:
            self.assertIsNotNone(self.udsclient.last_response, 'Client never received the PendingResponse message')
            self.completed = True
            diff = time.time() - t1
            self.assertGreater(diff, timeout, 'Timeout raised after %.3f seconds when it should be %.3f sec' % (diff, timeout))
            self.assertLess(diff, timeout+0.5, 'Timeout raised after %.3f seconds when it should be %.3f sec' % (diff, timeout))


    def test_payload_override_literal(self):
        request = self.conn.touserqueue.get(timeout=0.2)
        self.assertEqual(request, b'\x12\x34\x56\x78')
        self.conn.fromuserqueue.put(b"\x7E\x00")

    def _test_payload_override_literal(self):
        req = Request(service = services.TesterPresent, subfunction=0) 
        with self.udsclient.payload_override(b'\x12\x34\x56\x78'):
            response = self.udsclient.send_request(req)
            self.assertEqual(response.original_payload, b'\x7E\x00')


    def test_payload_override_func(self):
        request = self.conn.touserqueue.get(timeout=0.2)
        self.assertEqual(request, b'\x99\x88\x77\x66')
        self.conn.fromuserqueue.put(b"\x7E\x00")

    def _test_payload_override_func(self):
        def func(payload):
            return b'\x99\x88\x77\x66'

        req = Request(service = services.TesterPresent, subfunction=0) 
        with self.udsclient.payload_override(func):
            response = self.udsclient.send_request(req)
            self.assertEqual(response.original_payload, b'\x7E\x00')


from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *
import struct

class TransferData(BaseService):
    _sid = 0x36
    _use_subfunction = False

    supported_negative_response = [	 Response.Code.IncorrectMessageLegthOrInvalidFormat,
                                                    Response.Code.RequestSequenceError,
                                                    Response.Code.RequestOutOfRange,
                                                    Response.Code.TransferDataSuspended,
                                                    Response.Code.GeneralProgrammingFailure,
                                                    Response.Code.WrongBlockSequenceCounter,
                                                    Response.Code.VoltageTooHigh,
                                                    Response.Code.VoltageTooLow
                                                    ]

    @classmethod
    def make_request(cls, sequence_number, data=None):
        """
        Generates a request for TransferData

        :param sequence_number: Corresponds to an 8bit counter that should increment for each new block transferred.
                Allowed values are from 0 to 0xFF
        :type sequence_number: int

        :param data: Optional additional data to send to the server
        :type data: bytes

        :raises ValueError: If parameters are out of range, missing or wrong type
        """		
        from udsoncan import Request, MemoryLocation

        ServiceHelper.validate_int(sequence_number, min=0, max=0xFF, name='Block sequence counter')	# Not a subfunction!

        if data is not None and not isinstance(data, bytes):
            raise ValueError('data must be a bytes object')

        request = Request(service=cls)
        request.data = struct.pack('B', sequence_number)

        if data is not None:
            request.data += data
        return request

    @classmethod
    def interpret_response(cls, response):
        """
        Populates the response ``service_data`` property with an instance of :class:`TransferData.ResponseData<udsoncan.services.TransferData.ResponseData>`

        :param response: The received response to interpret
        :type response: :ref:`Response<Response>`

        :raises InvalidResponseException: If length of ``response.data`` is too short
        """		
        if len(response.data) < 1:
            raise InvalidResponseException(response, "Response data must be at least 1 bytes")

        response.service_data = cls.ResponseData()
        response.service_data.sequence_number_echo = response.data[0]
        response.service_data.parameter_records = response.data[1:] if len(response.data) > 1 else b''

    class ResponseData(BaseResponseData):
        """
        .. data:: sequence_number_echo

                Requests subfunction echoed back by the server

        .. data:: parameter_records

                Optional additional data associated with the response.
        """		
        def __init__(self):
            super().__init__(TransferData)
            self.sequence_number_echo = None
            self.parameter_records = None



from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *

class RequestTransferExit(BaseService):
    _sid = 0x37
    _use_subfunction = False
    _no_response_data = True

    supported_negative_response = [	 Response.Code.IncorrectMessageLegthOrInvalidFormat,
                                                    Response.Code.RequestSequenceError
                                                    ]

    @classmethod
    def make_request(cls, data=None):
        """
        Generates a request for RequestTransferExit

        :param data: Additional optional data to send to the server
        :type data: bytes

        :raises ValueError: If parameters are out of range, missing or wrong type
        """			
        from udsoncan import Request, MemoryLocation

        if data is not None and not isinstance(data, bytes):
            raise ValueError('data must be a bytes object')

        request = Request(service=cls, data=data)
        return request

    @classmethod
    def interpret_response(cls, response):
        """
        Populates the response ``service_data`` property with an instance of :class:`RequestTransferExit.ResponseData<udsoncan.services.RequestTransferExit.ResponseData>`

        :param response: The received response to interpret
        :type response: :ref:`Response<Response>`
        """				
        response.service_data = cls.ResponseData()
        response.service_data.parameter_records = response.data

    class ResponseData(BaseResponseData):
        """
        .. data:: parameter_records

                bytes object containing optional data provided by the server
        """		
        def __init__(self):
            super().__init__(RequestTransferExit)
            self.parameter_records = None



from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *
import struct

class RoutineControl(BaseService):
    _sid = 0x31

    class ControlType(BaseSubfunction):
        """
        RoutineControl defined subfunctions
        """		
        __pretty_name__ = 'control type'

        startRoutine = 1
        stopRoutine = 2
        requestRoutineResults = 3

    supported_negative_response = [	 Response.Code.SubFunctionNotSupported,
                                                    Response.Code.IncorrectMessageLegthOrInvalidFormat,
                                                    Response.Code.ConditionsNotCorrect,
                                                    Response.Code.RequestSequenceError,
                                                    Response.Code.RequestOutOfRange,
                                                    Response.Code.SecurityAccessDenied,
                                                    Response.Code.GeneralProgrammingFailure
                                                    ]

    @classmethod
    def make_request(cls, routine_id, control_type, data=None):
        """
        Generates a request for RoutineControl

        :param routine_id: The routine ID. Value should be between 0 and 0xFFFF
        :type routine_id: int

        :param control_type: Service subfunction. Allowed values are from 0 to 0x7F
        :type control_type: bytes

        :param data: Optional additional data to provide to the server
        :type data: bytes

        :raises ValueError: If parameters are out of range, missing or wrong type
        """		
        from udsoncan import Request

        ServiceHelper.validate_int(routine_id, min=0, max=0xFFFF, name='Routine ID')
        ServiceHelper.validate_int(control_type, min=0, max=0x7F, name='Routine control type')

        if data is not None:
            if not isinstance(data, bytes):
                raise ValueError('data must be a valid bytes object')

        request = Request(service=cls, subfunction=control_type)
        request.data = struct.pack('>H', routine_id)
        if data is not None:
            request.data += data

        return request

    @classmethod
    def interpret_response(cls, response):
        """
        Populates the response ``service_data`` property with an instance of :class:`RoutineControl.ResponseData<udsoncan.services.RoutineControl.ResponseData>`

        :param response: The received response to interpret
        :type response: :ref:`Response<Response>`

        :raises InvalidResponseException: If length of ``response.data`` is too short
        """

        if len(response.data) < 3: 	
            raise InvalidResponseException(response, "Response data must be at least 3 bytes")

        response.service_data = cls.ResponseData()
        response.service_data.control_type_echo = response.data[0]
        response.service_data.routine_id_echo = struct.unpack(">H", response.data[1:3])[0]
        response.service_data.routine_status_record = response.data[3:] if len(response.data) >3 else b''

    class ResponseData(BaseResponseData):
        """
        .. data:: control_type_echo

                Requests subfunction echoed back by the server

        .. data:: routine_id_echo

                Requests routine ID echoed back by the server.

        .. data:: routine_status_record

                Additional data associated with the response.
        """		
        def __init__(self):
            super().__init__(RoutineControl)

            self.control_type_echo = None
            self.routine_id_echo = None
            self.routine_status_record = None




from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *

class DiagnosticSessionControl(BaseService):
    _sid = 0x10

    supported_negative_response = [	Response.Code.SubFunctionNotSupported, 
                                                                    Response.Code.IncorrectMessageLegthOrInvalidFormat,
                                                                    Response.Code.ConditionsNotCorrect
                                                                    ]
    class Session(BaseSubfunction):
        """
        DiagnosticSessionControl defined subfunctions
        """		
        __pretty_name__ = 'session'	

        defaultSession = 1
        programmingSession = 2
        extendedDiagnosticSession = 3
        safetySystemDiagnosticSession = 4

    @classmethod
    def make_request(cls, session):
        """
        Generates a request for DiagnosticSessionControl service

        :param session: Service subfunction. Allowed values are from 0 to 0x7F
        :type session: int

        :raises ValueError: If parameters are out of range, missing or wrong type
        """

        from udsoncan import Request
        ServiceHelper.validate_int(session, min=0, max=0x7F, name='Session number')
        return Request(service=cls, subfunction=session)

    @classmethod
    def interpret_response(cls, response):
        """
        Populates the response ``service_data`` property with an instance of :class:`DiagnosticSessionControl.ResponseData<udsoncan.services.DiagnosticSessionControl.ResponseData>`

        :param response: The received response to interpret
        :type response: :ref:`Response<Response>`

        :raises InvalidResponseException: If length of ``response.data`` is too short
        """

        if len(response.data) < 1: 	# Should not happen as response decoder will raise an exception.
            raise InvalidResponseException(response, "Response data must be at least 1 bytes")

        response.service_data = cls.ResponseData()
        response.service_data.session_echo = response.data[0]
        response.service_data.session_param_records = response.data[1:] if len(response.data) > 1 else b''

        return response

    class ResponseData(BaseResponseData):
        """
        .. data:: session_echo

                Request subfunction echoed back by the server

        .. data:: session_param_records

                Additional data associated with the response.
        """		
        def __init__(self):
            super().__init__(DiagnosticSessionControl)
            self.session_echo = None
            self.session_param_records = None



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name = 'udsoncan',
  packages = find_packages(exclude=['test']),
  package_data={
      '': ['*.conf'],
  },
  version = '1.12.1',
  description = 'Implementation of the Unified Diagnostic Service (UDS) protocol (ISO-14229) used in the automotive industry.',
  long_description=long_description,
  author = 'Pier-Yves Lessard',
  author_email = 'py.lessard@gmail.com',
  license='MIT',
  url = 'https://github.com/pylessard/python-udsoncan',
  download_url = 'https://github.com/pylessard/python-udsoncan/archive/v1.12.1.tar.gz',
  keywords = ['uds', '14229', 'iso-14229', 'diagnostic', 'automotive'], 
  python_requires='>=3.0',
  classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Operating System :: POSIX :: Linux",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator",
        ],
)
